#
# Copyright (C) 2005-2008 by Pieter Palmers
# Copyright (C) 2008 by Jonathan Woithe
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtCore import Qt
# from PyQt4.QtGui import QWidget, QApplication
from ffado.import_pyqt import *

from ffado.config import *

import logging
log = logging.getLogger('motu')

# Model defines.  These must agree with what is used in motu_avdevice.h.
MOTU_MODEL_NONE             = 0x0000
MOTU_MODEL_828mkII          = 0x0001
MOTU_MODEL_TRAVELER         = 0x0002
MOTU_MODEL_ULTRALITE        = 0x0003
MOTU_MODEL_8PRE             = 0x0004
MOTU_MODEL_828MkI           = 0x0005
MOTU_MODEL_896HD            = 0x0006
MOTU_MODEL_828mk3           = 0x0007
MOTU_MODEL_ULTRALITEmk3     = 0x0008
MOTU_MODEL_ULTRALITEmk3_HYB = 0x0009
MOTU_MODEL_TRAVELERmk3      = 0x000a
MOTU_MODEL_896HDmk3         = 0x000b

class Motu(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/motu", self)

        self.init()

    def init(self):

        # For matrix mixer controls (channel faders, pans, solos, mutes) the
        # first index (the row) is the zero-based mix number while the
        # second index (the column) is the zero-based channel number.  The
        # third index gives the control of the stereo pair of the control
        # used as the key.  The order of the channel enumeration must agree
        # with that used when creating the dbus controls within
        # motu_avdevice.cpp.
        self.ChannelFaders={
            self.mix1ana1_fader: ['/Mixer/fader', 0, 0, self.mix1ana2_fader],
            self.mix1ana2_fader: ['/Mixer/fader', 0, 1, self.mix1ana1_fader],
            self.mix1ana3_fader: ['/Mixer/fader', 0, 2, self.mix1ana4_fader],
            self.mix1ana4_fader: ['/Mixer/fader', 0, 3, self.mix1ana3_fader],
            self.mix1ana5_fader: ['/Mixer/fader', 0, 4, self.mix1ana6_fader],
            self.mix1ana6_fader: ['/Mixer/fader', 0, 5, self.mix1ana5_fader],
            self.mix1ana7_fader: ['/Mixer/fader', 0, 6, self.mix1ana8_fader],
            self.mix1ana8_fader: ['/Mixer/fader', 0, 7, self.mix1ana7_fader],
            self.mix1aes1_fader: ['/Mixer/fader', 0, 8, self.mix1aes2_fader],
            self.mix1aes2_fader: ['/Mixer/fader', 0, 9, self.mix1aes1_fader],
            self.mix1spdif1_fader: ['/Mixer/fader', 0, 10, self.mix1spdif2_fader],
            self.mix1spdif2_fader: ['/Mixer/fader', 0, 11, self.mix1spdif1_fader],
            self.mix1adat1_fader: ['/Mixer/fader', 0, 12, self.mix1adat2_fader],
            self.mix1adat2_fader: ['/Mixer/fader', 0, 13, self.mix1adat1_fader],
            self.mix1adat3_fader: ['/Mixer/fader', 0, 14, self.mix1adat4_fader],
            self.mix1adat4_fader: ['/Mixer/fader', 0, 15, self.mix1adat3_fader],
            self.mix1adat5_fader: ['/Mixer/fader', 0, 16, self.mix1adat6_fader],
            self.mix1adat6_fader: ['/Mixer/fader', 0, 17, self.mix1adat5_fader],
            self.mix1adat7_fader: ['/Mixer/fader', 0, 18, self.mix1adat8_fader],
            self.mix1adat8_fader: ['/Mixer/fader', 0, 19, self.mix1adat7_fader],

            self.mix2ana1_fader: ['/Mixer/fader', 1, 0, self.mix2ana2_fader],
            self.mix2ana2_fader: ['/Mixer/fader', 1, 1, self.mix2ana1_fader],
            self.mix2ana3_fader: ['/Mixer/fader', 1, 2, self.mix2ana4_fader],
            self.mix2ana4_fader: ['/Mixer/fader', 1, 3, self.mix2ana3_fader],
            self.mix2ana5_fader: ['/Mixer/fader', 1, 4, self.mix2ana6_fader],
            self.mix2ana6_fader: ['/Mixer/fader', 1, 5, self.mix2ana5_fader],
            self.mix2ana7_fader: ['/Mixer/fader', 1, 6, self.mix2ana8_fader],
            self.mix2ana8_fader: ['/Mixer/fader', 1, 7, self.mix2ana7_fader],
            self.mix2aes1_fader: ['/Mixer/fader', 1, 8, self.mix2aes2_fader],
            self.mix2aes2_fader: ['/Mixer/fader', 1, 9, self.mix2aes1_fader],
            self.mix2spdif1_fader: ['/Mixer/fader', 1, 10, self.mix2spdif2_fader],
            self.mix2spdif2_fader: ['/Mixer/fader', 1, 11, self.mix2spdif1_fader],
            self.mix2adat1_fader: ['/Mixer/fader', 1, 12, self.mix2adat2_fader],
            self.mix2adat2_fader: ['/Mixer/fader', 1, 13, self.mix2adat1_fader],
            self.mix2adat3_fader: ['/Mixer/fader', 1, 14, self.mix2adat4_fader],
            self.mix2adat4_fader: ['/Mixer/fader', 1, 15, self.mix2adat3_fader],
            self.mix2adat5_fader: ['/Mixer/fader', 1, 16, self.mix2adat6_fader],
            self.mix2adat6_fader: ['/Mixer/fader', 1, 17, self.mix2adat5_fader],
            self.mix2adat7_fader: ['/Mixer/fader', 1, 18, self.mix2adat8_fader],
            self.mix2adat8_fader: ['/Mixer/fader', 1, 19, self.mix2adat7_fader],

            self.mix3ana1_fader: ['/Mixer/fader', 2, 0, self.mix3ana2_fader],
            self.mix3ana2_fader: ['/Mixer/fader', 2, 1, self.mix3ana1_fader],
            self.mix3ana3_fader: ['/Mixer/fader', 2, 2, self.mix3ana4_fader],
            self.mix3ana4_fader: ['/Mixer/fader', 2, 3, self.mix3ana3_fader],
            self.mix3ana5_fader: ['/Mixer/fader', 2, 4, self.mix3ana6_fader],
            self.mix3ana6_fader: ['/Mixer/fader', 2, 5, self.mix3ana5_fader],
            self.mix3ana7_fader: ['/Mixer/fader', 2, 6, self.mix3ana8_fader],
            self.mix3ana8_fader: ['/Mixer/fader', 2, 7, self.mix3ana7_fader],
            self.mix3aes1_fader: ['/Mixer/fader', 2, 8, self.mix3aes2_fader],
            self.mix3aes2_fader: ['/Mixer/fader', 2, 9, self.mix3aes1_fader],
            self.mix3spdif1_fader: ['/Mixer/fader', 2, 10, self.mix3spdif2_fader],
            self.mix3spdif2_fader: ['/Mixer/fader', 2, 11, self.mix3spdif1_fader],
            self.mix3adat1_fader: ['/Mixer/fader', 2, 12, self.mix3adat2_fader],
            self.mix3adat2_fader: ['/Mixer/fader', 2, 13, self.mix3adat1_fader],
            self.mix3adat3_fader: ['/Mixer/fader', 2, 14, self.mix3adat4_fader],
            self.mix3adat4_fader: ['/Mixer/fader', 2, 15, self.mix3adat3_fader],
            self.mix3adat5_fader: ['/Mixer/fader', 2, 16, self.mix3adat6_fader],
            self.mix3adat6_fader: ['/Mixer/fader', 2, 17, self.mix3adat5_fader],
            self.mix3adat7_fader: ['/Mixer/fader', 2, 18, self.mix3adat8_fader],
            self.mix3adat8_fader: ['/Mixer/fader', 2, 19, self.mix3adat7_fader],

            self.mix4ana1_fader: ['/Mixer/fader', 3, 0, self.mix4ana2_fader],
            self.mix4ana2_fader: ['/Mixer/fader', 3, 1, self.mix4ana1_fader],
            self.mix4ana3_fader: ['/Mixer/fader', 3, 2, self.mix4ana4_fader],
            self.mix4ana4_fader: ['/Mixer/fader', 3, 3, self.mix4ana3_fader],
            self.mix4ana5_fader: ['/Mixer/fader', 3, 4, self.mix4ana6_fader],
            self.mix4ana6_fader: ['/Mixer/fader', 3, 5, self.mix4ana5_fader],
            self.mix4ana7_fader: ['/Mixer/fader', 3, 6, self.mix4ana8_fader],
            self.mix4ana8_fader: ['/Mixer/fader', 3, 7, self.mix4ana7_fader],
            self.mix4aes1_fader: ['/Mixer/fader', 3, 8, self.mix4aes2_fader],
            self.mix4aes2_fader: ['/Mixer/fader', 3, 9, self.mix4aes1_fader],
            self.mix4spdif1_fader: ['/Mixer/fader', 3, 10, self.mix4spdif2_fader],
            self.mix4spdif2_fader: ['/Mixer/fader', 3, 11, self.mix4spdif1_fader],
            self.mix4adat1_fader: ['/Mixer/fader', 3, 12, self.mix4adat2_fader],
            self.mix4adat2_fader: ['/Mixer/fader', 3, 13, self.mix4adat1_fader],
            self.mix4adat3_fader: ['/Mixer/fader', 3, 14, self.mix4adat4_fader],
            self.mix4adat4_fader: ['/Mixer/fader', 3, 15, self.mix4adat3_fader],
            self.mix4adat5_fader: ['/Mixer/fader', 3, 16, self.mix4adat6_fader],
            self.mix4adat6_fader: ['/Mixer/fader', 3, 17, self.mix4adat5_fader],
            self.mix4adat7_fader: ['/Mixer/fader', 3, 18, self.mix4adat8_fader],
            self.mix4adat8_fader: ['/Mixer/fader', 3, 19, self.mix4adat7_fader],
        }

        self.Faders={
            self.mix1_fader: ['/Mixer/Mix1/Mix_fader'],
            self.mix2_fader: ['/Mixer/Mix2/Mix_fader'],
            self.mix3_fader: ['/Mixer/Mix3/Mix_fader'],
            self.mix4_fader: ['/Mixer/Mix4/Mix_fader'],
            self.mainout_fader: ['/Mixer/Mainout_fader'],
            self.phones_fader:  ['/Mixer/Phones_fader'],
        }

        self.ChannelControls={
            self.mix1ana1_pan:   ['/Mixer/pan', 0, 0],
            self.mix1ana2_pan:   ['/Mixer/pan', 0, 1],
            self.mix1ana3_pan:   ['/Mixer/pan', 0, 2],
            self.mix1ana4_pan:   ['/Mixer/pan', 0, 3],
            self.mix1ana5_pan:   ['/Mixer/pan', 0, 4],
            self.mix1ana6_pan:   ['/Mixer/pan', 0, 5],
            self.mix1ana7_pan:   ['/Mixer/pan', 0, 6],
            self.mix1ana8_pan:   ['/Mixer/pan', 0, 7],
            self.mix1aes1_pan:   ['/Mixer/pan', 0, 8],
            self.mix1aes2_pan:   ['/Mixer/pan', 0, 9],
            self.mix1spdif1_pan: ['/Mixer/pan', 0, 10],
            self.mix1spdif2_pan: ['/Mixer/pan', 0, 11],
            self.mix1adat1_pan:  ['/Mixer/pan', 0, 12],
            self.mix1adat2_pan:  ['/Mixer/pan', 0, 13],
            self.mix1adat3_pan:  ['/Mixer/pan', 0, 14],
            self.mix1adat4_pan:  ['/Mixer/pan', 0, 15],
            self.mix1adat5_pan:  ['/Mixer/pan', 0, 16],
            self.mix1adat6_pan:  ['/Mixer/pan', 0, 17],
            self.mix1adat7_pan:  ['/Mixer/pan', 0, 18],
            self.mix1adat8_pan:  ['/Mixer/pan', 0, 19],

            self.mix2ana1_pan:   ['/Mixer/pan', 1, 0],
            self.mix2ana2_pan:   ['/Mixer/pan', 1, 1],
            self.mix2ana3_pan:   ['/Mixer/pan', 1, 2],
            self.mix2ana4_pan:   ['/Mixer/pan', 1, 3],
            self.mix2ana5_pan:   ['/Mixer/pan', 1, 4],
            self.mix2ana6_pan:   ['/Mixer/pan', 1, 5],
            self.mix2ana7_pan:   ['/Mixer/pan', 1, 6],
            self.mix2ana8_pan:   ['/Mixer/pan', 1, 7],
            self.mix2aes1_pan:   ['/Mixer/pan', 1, 8],
            self.mix2aes2_pan:   ['/Mixer/pan', 1, 9],
            self.mix2spdif1_pan: ['/Mixer/pan', 1, 10],
            self.mix2spdif2_pan: ['/Mixer/pan', 1, 11],
            self.mix2adat1_pan:  ['/Mixer/pan', 1, 12],
            self.mix2adat2_pan:  ['/Mixer/pan', 1, 13],
            self.mix2adat3_pan:  ['/Mixer/pan', 1, 14],
            self.mix2adat4_pan:  ['/Mixer/pan', 1, 15],
            self.mix2adat5_pan:  ['/Mixer/pan', 1, 16],
            self.mix2adat6_pan:  ['/Mixer/pan', 1, 17],
            self.mix2adat7_pan:  ['/Mixer/pan', 1, 18],
            self.mix2adat8_pan:  ['/Mixer/pan', 1, 19],

            self.mix3ana1_pan:   ['/Mixer/pan', 2, 0],
            self.mix3ana2_pan:   ['/Mixer/pan', 2, 1],
            self.mix3ana3_pan:   ['/Mixer/pan', 2, 2],
            self.mix3ana4_pan:   ['/Mixer/pan', 2, 3],
            self.mix3ana5_pan:   ['/Mixer/pan', 2, 4],
            self.mix3ana6_pan:   ['/Mixer/pan', 2, 5],
            self.mix3ana7_pan:   ['/Mixer/pan', 2, 6],
            self.mix3ana8_pan:   ['/Mixer/pan', 2, 7],
            self.mix3aes1_pan:   ['/Mixer/pan', 2, 8],
            self.mix3aes2_pan:   ['/Mixer/pan', 2, 9],
            self.mix3spdif1_pan: ['/Mixer/pan', 2, 10],
            self.mix3spdif2_pan: ['/Mixer/pan', 2, 11],
            self.mix3adat1_pan:  ['/Mixer/pan', 2, 12],
            self.mix3adat2_pan:  ['/Mixer/pan', 2, 13],
            self.mix3adat3_pan:  ['/Mixer/pan', 2, 14],
            self.mix3adat4_pan:  ['/Mixer/pan', 2, 15],
            self.mix3adat5_pan:  ['/Mixer/pan', 2, 16],
            self.mix3adat6_pan:  ['/Mixer/pan', 2, 17],
            self.mix3adat7_pan:  ['/Mixer/pan', 2, 18],
            self.mix3adat8_pan:  ['/Mixer/pan', 2, 19],

            self.mix4ana1_pan:   ['/Mixer/pan', 3, 0],
            self.mix4ana2_pan:   ['/Mixer/pan', 3, 1],
            self.mix4ana3_pan:   ['/Mixer/pan', 3, 2],
            self.mix4ana4_pan:   ['/Mixer/pan', 3, 3],
            self.mix4ana5_pan:   ['/Mixer/pan', 3, 4],
            self.mix4ana6_pan:   ['/Mixer/pan', 3, 5],
            self.mix4ana7_pan:   ['/Mixer/pan', 3, 6],
            self.mix4ana8_pan:   ['/Mixer/pan', 3, 7],
            self.mix4aes1_pan:   ['/Mixer/pan', 3, 8],
            self.mix4aes2_pan:   ['/Mixer/pan', 3, 9],
            self.mix4spdif1_pan: ['/Mixer/pan', 3, 10],
            self.mix4spdif2_pan: ['/Mixer/pan', 3, 11],
            self.mix4adat1_pan:  ['/Mixer/pan', 3, 12],
            self.mix4adat2_pan:  ['/Mixer/pan', 3, 13],
            self.mix4adat3_pan:  ['/Mixer/pan', 3, 14],
            self.mix4adat4_pan:  ['/Mixer/pan', 3, 15],
            self.mix4adat5_pan:  ['/Mixer/pan', 3, 16],
            self.mix4adat6_pan:  ['/Mixer/pan', 3, 17],
            self.mix4adat7_pan:  ['/Mixer/pan', 3, 18],
            self.mix4adat8_pan:  ['/Mixer/pan', 3, 19],
        }

        self.Controls={
            self.ana1_trimgain:  ['/Mixer/Control/Ana1_trimgain'],
            self.ana2_trimgain:  ['/Mixer/Control/Ana2_trimgain'],
            self.ana3_trimgain:  ['/Mixer/Control/Ana3_trimgain'],
            self.ana4_trimgain:  ['/Mixer/Control/Ana4_trimgain'],
            self.ana5_trimgain:  ['/Mixer/Control/Ana5_trimgain'],
            self.ana6_trimgain:  ['/Mixer/Control/Ana6_trimgain'],
            self.ana7_trimgain:  ['/Mixer/Control/Ana7_trimgain'],
            self.ana8_trimgain:  ['/Mixer/Control/Ana8_trimgain'],
            self.spdif1_trimgain:  ['/Mixer/Control/Spdif1_trimgain'],
            self.spdif2_trimgain:  ['/Mixer/Control/Spdif2_trimgain'],
        }

        self.ChannelBinarySwitches={
            self.mix1ana1_mute:  ['/Mixer/mute', 0, 0],
            self.mix1ana2_mute:  ['/Mixer/mute', 0, 1],
            self.mix1ana3_mute:  ['/Mixer/mute', 0, 2],
            self.mix1ana4_mute:  ['/Mixer/mute', 0, 3],
            self.mix1ana5_mute:  ['/Mixer/mute', 0, 4],
            self.mix1ana6_mute:  ['/Mixer/mute', 0, 5],
            self.mix1ana7_mute:  ['/Mixer/mute', 0, 6],
            self.mix1ana8_mute:  ['/Mixer/mute', 0, 7],
            self.mix1aes1_mute:  ['/Mixer/mute', 0, 8],
            self.mix1aes2_mute:  ['/Mixer/mute', 0, 9],
            self.mix1spdif1_mute: ['/Mixer/mute', 0, 10],
            self.mix1spdif2_mute: ['/Mixer/mute', 0, 11],
            self.mix1adat1_mute: ['/Mixer/mute', 0, 12],
            self.mix1adat2_mute: ['/Mixer/mute', 0, 13],
            self.mix1adat3_mute: ['/Mixer/mute', 0, 14],
            self.mix1adat4_mute: ['/Mixer/mute', 0, 15],
            self.mix1adat5_mute: ['/Mixer/mute', 0, 16],
            self.mix1adat6_mute: ['/Mixer/mute', 0, 17],
            self.mix1adat7_mute: ['/Mixer/mute', 0, 18],
            self.mix1adat8_mute: ['/Mixer/mute', 0, 19],
            self.mix1ana1_solo:  ['/Mixer/solo', 0, 0],
            self.mix1ana2_solo:  ['/Mixer/solo', 0, 1],
            self.mix1ana3_solo:  ['/Mixer/solo', 0, 2],
            self.mix1ana4_solo:  ['/Mixer/solo', 0, 3],
            self.mix1ana5_solo:  ['/Mixer/solo', 0, 4],
            self.mix1ana6_solo:  ['/Mixer/solo', 0, 5],
            self.mix1ana7_solo:  ['/Mixer/solo', 0, 6],
            self.mix1ana8_solo:  ['/Mixer/solo', 0, 7],
            self.mix1aes1_solo:  ['/Mixer/solo', 0, 8],
            self.mix1aes2_solo:  ['/Mixer/solo', 0, 9],
            self.mix1spdif1_solo: ['/Mixer/solo', 0, 10],
            self.mix1spdif2_solo: ['/Mixer/solo', 0, 11],
            self.mix1adat1_solo: ['/Mixer/solo', 0, 12],
            self.mix1adat2_solo: ['/Mixer/solo', 0, 13],
            self.mix1adat3_solo: ['/Mixer/solo', 0, 14],
            self.mix1adat4_solo: ['/Mixer/solo', 0, 15],
            self.mix1adat5_solo: ['/Mixer/solo', 0, 16],
            self.mix1adat6_solo: ['/Mixer/solo', 0, 17],
            self.mix1adat7_solo: ['/Mixer/solo', 0, 18],
            self.mix1adat8_solo: ['/Mixer/solo', 0, 19],

            self.mix2ana1_mute:  ['/Mixer/mute', 1, 0],
            self.mix2ana2_mute:  ['/Mixer/mute', 1, 1],
            self.mix2ana3_mute:  ['/Mixer/mute', 1, 2],
            self.mix2ana4_mute:  ['/Mixer/mute', 1, 3],
            self.mix2ana5_mute:  ['/Mixer/mute', 1, 4],
            self.mix2ana6_mute:  ['/Mixer/mute', 1, 5],
            self.mix2ana7_mute:  ['/Mixer/mute', 1, 6],
            self.mix2ana8_mute:  ['/Mixer/mute', 1, 7],
            self.mix2aes1_mute:  ['/Mixer/mute', 1, 8],
            self.mix2aes2_mute:  ['/Mixer/mute', 1, 9],
            self.mix2spdif1_mute: ['/Mixer/mute', 1, 10],
            self.mix2spdif2_mute: ['/Mixer/mute', 1, 11],
            self.mix2adat1_mute: ['/Mixer/mute', 1, 12],
            self.mix2adat2_mute: ['/Mixer/mute', 1, 13],
            self.mix2adat3_mute: ['/Mixer/mute', 1, 14],
            self.mix2adat4_mute: ['/Mixer/mute', 1, 15],
            self.mix2adat5_mute: ['/Mixer/mute', 1, 16],
            self.mix2adat6_mute: ['/Mixer/mute', 1, 17],
            self.mix2adat7_mute: ['/Mixer/mute', 1, 18],
            self.mix2adat8_mute: ['/Mixer/mute', 1, 19],
            self.mix2ana1_solo:  ['/Mixer/solo', 1, 0],
            self.mix2ana2_solo:  ['/Mixer/solo', 1, 1],
            self.mix2ana3_solo:  ['/Mixer/solo', 1, 2],
            self.mix2ana4_solo:  ['/Mixer/solo', 1, 3],
            self.mix2ana5_solo:  ['/Mixer/solo', 1, 4],
            self.mix2ana6_solo:  ['/Mixer/solo', 1, 5],
            self.mix2ana7_solo:  ['/Mixer/solo', 1, 6],
            self.mix2ana8_solo:  ['/Mixer/solo', 1, 7],
            self.mix2aes1_solo:  ['/Mixer/solo', 1, 8],
            self.mix2aes2_solo:  ['/Mixer/solo', 1, 9],
            self.mix2spdif1_solo: ['/Mixer/solo', 1, 10],
            self.mix2spdif2_solo: ['/Mixer/solo', 1, 11],
            self.mix2adat1_solo: ['/Mixer/solo', 1, 12],
            self.mix2adat2_solo: ['/Mixer/solo', 1, 13],
            self.mix2adat3_solo: ['/Mixer/solo', 1, 14],
            self.mix2adat4_solo: ['/Mixer/solo', 1, 15],
            self.mix2adat5_solo: ['/Mixer/solo', 1, 16],
            self.mix2adat6_solo: ['/Mixer/solo', 1, 17],
            self.mix2adat7_solo: ['/Mixer/solo', 1, 18],
            self.mix2adat8_solo: ['/Mixer/solo', 1, 19],

            self.mix3ana1_mute:  ['/Mixer/mute', 2, 0],
            self.mix3ana2_mute:  ['/Mixer/mute', 2, 1],
            self.mix3ana3_mute:  ['/Mixer/mute', 2, 2],
            self.mix3ana4_mute:  ['/Mixer/mute', 2, 3],
            self.mix3ana5_mute:  ['/Mixer/mute', 2, 4],
            self.mix3ana6_mute:  ['/Mixer/mute', 2, 5],
            self.mix3ana7_mute:  ['/Mixer/mute', 2, 6],
            self.mix3ana8_mute:  ['/Mixer/mute', 2, 7],
            self.mix3aes1_mute:  ['/Mixer/mute', 2, 8],
            self.mix3aes2_mute:  ['/Mixer/mute', 2, 9],
            self.mix3spdif1_mute: ['/Mixer/mute', 2, 10],
            self.mix3spdif2_mute: ['/Mixer/mute', 2, 11],
            self.mix3adat1_mute: ['/Mixer/mute', 2, 12],
            self.mix3adat2_mute: ['/Mixer/mute', 2, 13],
            self.mix3adat3_mute: ['/Mixer/mute', 2, 14],
            self.mix3adat4_mute: ['/Mixer/mute', 2, 15],
            self.mix3adat5_mute: ['/Mixer/mute', 2, 16],
            self.mix3adat6_mute: ['/Mixer/mute', 2, 17],
            self.mix3adat7_mute: ['/Mixer/mute', 2, 18],
            self.mix3adat8_mute: ['/Mixer/mute', 2, 19],
            self.mix3ana1_solo:  ['/Mixer/solo', 2, 0],
            self.mix3ana2_solo:  ['/Mixer/solo', 2, 1],
            self.mix3ana3_solo:  ['/Mixer/solo', 2, 2],
            self.mix3ana4_solo:  ['/Mixer/solo', 2, 3],
            self.mix3ana5_solo:  ['/Mixer/solo', 2, 4],
            self.mix3ana6_solo:  ['/Mixer/solo', 2, 5],
            self.mix3ana7_solo:  ['/Mixer/solo', 2, 6],
            self.mix3ana8_solo:  ['/Mixer/solo', 2, 7],
            self.mix3aes1_solo:  ['/Mixer/solo', 2, 8],
            self.mix3aes2_solo:  ['/Mixer/solo', 2, 9],
            self.mix3spdif1_solo: ['/Mixer/solo', 2, 10],
            self.mix3spdif2_solo: ['/Mixer/solo', 2, 11],
            self.mix3adat1_solo: ['/Mixer/solo', 2, 12],
            self.mix3adat2_solo: ['/Mixer/solo', 2, 13],
            self.mix3adat3_solo: ['/Mixer/solo', 2, 14],
            self.mix3adat4_solo: ['/Mixer/solo', 2, 15],
            self.mix3adat5_solo: ['/Mixer/solo', 2, 16],
            self.mix3adat6_solo: ['/Mixer/solo', 2, 17],
            self.mix3adat7_solo: ['/Mixer/solo', 2, 18],
            self.mix3adat8_solo: ['/Mixer/solo', 2, 19],

            self.mix4ana1_mute:  ['/Mixer/mute', 3, 0],
            self.mix4ana2_mute:  ['/Mixer/mute', 3, 1],
            self.mix4ana3_mute:  ['/Mixer/mute', 3, 2],
            self.mix4ana4_mute:  ['/Mixer/mute', 3, 3],
            self.mix4ana5_mute:  ['/Mixer/mute', 3, 4],
            self.mix4ana6_mute:  ['/Mixer/mute', 3, 5],
            self.mix4ana7_mute:  ['/Mixer/mute', 3, 6],
            self.mix4ana8_mute:  ['/Mixer/mute', 3, 7],
            self.mix4aes1_mute:  ['/Mixer/mute', 3, 8],
            self.mix4aes2_mute:  ['/Mixer/mute', 3, 9],
            self.mix4spdif1_mute: ['/Mixer/mute', 3, 10],
            self.mix4spdif2_mute: ['/Mixer/mute', 3, 11],
            self.mix4adat1_mute: ['/Mixer/mute', 3, 12],
            self.mix4adat2_mute: ['/Mixer/mute', 3, 13],
            self.mix4adat3_mute: ['/Mixer/mute', 3, 14],
            self.mix4adat4_mute: ['/Mixer/mute', 3, 15],
            self.mix4adat5_mute: ['/Mixer/mute', 3, 16],
            self.mix4adat6_mute: ['/Mixer/mute', 3, 17],
            self.mix4adat7_mute: ['/Mixer/mute', 3, 18],
            self.mix4adat8_mute: ['/Mixer/mute', 3, 19],
            self.mix4ana1_solo:  ['/Mixer/solo', 3, 0],
            self.mix4ana2_solo:  ['/Mixer/solo', 3, 1],
            self.mix4ana3_solo:  ['/Mixer/solo', 3, 2],
            self.mix4ana4_solo:  ['/Mixer/solo', 3, 3],
            self.mix4ana5_solo:  ['/Mixer/solo', 3, 4],
            self.mix4ana6_solo:  ['/Mixer/solo', 3, 5],
            self.mix4ana7_solo:  ['/Mixer/solo', 3, 6],
            self.mix4ana8_solo:  ['/Mixer/solo', 3, 7],
            self.mix4aes1_solo:  ['/Mixer/solo', 3, 8],
            self.mix4aes2_solo:  ['/Mixer/solo', 3, 9],
            self.mix4spdif1_solo: ['/Mixer/solo', 3, 10],
            self.mix4spdif2_solo: ['/Mixer/solo', 3, 11],
            self.mix4adat1_solo: ['/Mixer/solo', 3, 12],
            self.mix4adat2_solo: ['/Mixer/solo', 3, 13],
            self.mix4adat3_solo: ['/Mixer/solo', 3, 14],
            self.mix4adat4_solo: ['/Mixer/solo', 3, 15],
            self.mix4adat5_solo: ['/Mixer/solo', 3, 16],
            self.mix4adat6_solo: ['/Mixer/solo', 3, 17],
            self.mix4adat7_solo: ['/Mixer/solo', 3, 18],
            self.mix4adat8_solo: ['/Mixer/solo', 3, 19],
        }

        self.BinarySwitches={
            self.mix1_mute:      ['/Mixer/Mix1/Mix_mute'],
            self.mix2_mute:      ['/Mixer/Mix2/Mix_mute'],
            self.mix3_mute:      ['/Mixer/Mix3/Mix_mute'],
            self.mix4_mute:      ['/Mixer/Mix4/Mix_mute'],

            self.ana1_pad:       ['/Mixer/Control/Ana1_pad'],
            self.ana2_pad:       ['/Mixer/Control/Ana2_pad'],
            self.ana3_pad:       ['/Mixer/Control/Ana3_pad'],
            self.ana4_pad:       ['/Mixer/Control/Ana4_pad'],
            self.ana5_pad:       ['/Mixer/Control/Ana5_pad'],
            self.ana6_pad:       ['/Mixer/Control/Ana6_pad'],
            self.ana7_pad:       ['/Mixer/Control/Ana7_pad'],
            self.ana8_pad:       ['/Mixer/Control/Ana8_pad'],
            self.ana1_invert:    ['/Mixer/Control/Ana1_invert'],
            self.ana2_invert:    ['/Mixer/Control/Ana2_invert'],
            self.ana3_invert:    ['/Mixer/Control/Ana3_invert'],
            self.ana4_invert:    ['/Mixer/Control/Ana4_invert'],
            self.ana5_invert:    ['/Mixer/Control/Ana5_invert'],
            self.ana6_invert:    ['/Mixer/Control/Ana6_invert'],
            self.ana7_invert:    ['/Mixer/Control/Ana7_invert'],
            self.ana8_invert:    ['/Mixer/Control/Ana8_invert'],
            self.spdif1_invert:  ['/Mixer/Control/Spdif1_invert'],
            self.spdif2_invert:  ['/Mixer/Control/Spdif2_invert'],
            self.ana1_level:     ['/Mixer/Control/Ana1_level'],
            self.ana2_level:     ['/Mixer/Control/Ana2_level'],
            self.ana3_level:     ['/Mixer/Control/Ana3_level'],
            self.ana4_level:     ['/Mixer/Control/Ana4_level'],
            self.ana5_level:     ['/Mixer/Control/Ana5_level'],
            self.ana6_level:     ['/Mixer/Control/Ana6_level'],
            self.ana7_level:     ['/Mixer/Control/Ana7_level'],
            self.ana8_level:     ['/Mixer/Control/Ana8_level'],
            self.ana1_boost:     ['/Mixer/Control/Ana1_boost'],
            self.ana2_boost:     ['/Mixer/Control/Ana2_boost'],
            self.ana3_boost:     ['/Mixer/Control/Ana3_boost'],
            self.ana4_boost:     ['/Mixer/Control/Ana4_boost'],
            self.ana5_boost:     ['/Mixer/Control/Ana5_boost'],
            self.ana6_boost:     ['/Mixer/Control/Ana6_boost'],
            self.ana7_boost:     ['/Mixer/Control/Ana7_boost'],
            self.ana8_boost:     ['/Mixer/Control/Ana8_boost'],
        }

        self.Selectors={
            self.mix1_dest:      ['/Mixer/Mix1/Mix_dest'],
            self.mix2_dest:      ['/Mixer/Mix2/Mix_dest'],
            self.mix3_dest:      ['/Mixer/Mix3/Mix_dest'],
            self.mix4_dest:      ['/Mixer/Mix4/Mix_dest'],

            self.phones_src:	 ['/Mixer/Control/Phones_src'],

            self.optical_in_mode:   ['/Mixer/Control/OpticalIn_mode'],
            self.optical_out_mode:  ['/Mixer/Control/OpticalOut_mode'],

            self.meter_src_ctrl:    ['/Mixer/Control/Meter_src'],
            self.aesebu_meter_ctrl: ['/Mixer/Control/Meter_aesebu_src'],
            self.peakhold_time_ctrl:['/Mixer/Control/Meter_peakhold_time'],
            self.cliphold_time_ctrl:['/Mixer/Control/Meter_cliphold_time'],
        }

        # Other mixer variables
        self.is_streaming = 0
        self.sample_rate = 0
        self.model = MOTU_MODEL_NONE


    # public slot: channel faders within a matrix mixer
    def updateChannelFader(self, a0):
        sender = self.sender()
        vol = a0
        log.debug("setting %s for mix %d channel %d to %d" % (self.ChannelFaders[sender][0], 
            self.ChannelFaders[sender][1], self.ChannelFaders[sender][2], vol))
        self.hw.setMatrixMixerValue(self.ChannelFaders[sender][0],
            self.ChannelFaders[sender][1], self.ChannelFaders[sender][2], vol)
        # Using the ctrl modifier key makes stereo pairs move in unison
        if (QApplication.keyboardModifiers() == Qt.ControlModifier):
            pair = self.ChannelFaders[sender][3]
            pair.setValue(vol)

    # public slot: a multivalue control within a matrix mixer
    def updateChannelControl(self, a0):
        sender = self.sender()
        val = a0
        log.debug("setting %s for mix %d channel %d to %d" % (self.ChannelControls[sender][0], 
            self.ChannelControls[sender][1], self.ChannelControls[sender][2], val))
        self.hw.setMatrixMixerValue(self.ChannelControls[sender][0], 
            self.ChannelControls[sender][1], self.ChannelControls[sender][2], val)

    # public slot: a generic single multivalue control
    def updateControl(self, a0):
        sender = self.sender()
        val = a0
        log.debug("setting %s control to %d" % (self.Controls[sender][0], val))
        self.hw.setDiscrete(self.Controls[sender][0], val)

    # public slot: a binary switch within a matrix mixer
    def updateChannelBinarySwitch(self, a0):
        sender = self.sender()
        val=a0
        log.debug("setting %s for mix %d channel %d switch to %d" % (self.ChannelBinarySwitches[sender][0], 
            self.ChannelBinarySwitches[sender][1], self.ChannelBinarySwitches[sender][2], val))
        self.hw.setMatrixMixerValue(self.ChannelBinarySwitches[sender][0], 
            self.ChannelBinarySwitches[sender][1], self.ChannelBinarySwitches[sender][2], val)

    # public slot: generic single binary switch
    def updateBinarySwitch(self, a0):
        sender = self.sender()
        val=a0
        log.debug("setting %s switch to %d" % (self.BinarySwitches[sender][0], val))
        self.hw.setDiscrete(self.BinarySwitches[sender][0], val)

    # public slot: a faders (not in a matrix mixer)
    def updateFader(self, a0):
        sender = self.sender()
        vol = a0
        log.debug("setting %s mix fader to %d" % (self.Faders[sender][0], vol))
        self.hw.setDiscrete(self.Faders[sender][0], vol)

    # public slot: selectors (eg: mix destination controls)
    def updateSelector(self, a0):
        sender = self.sender()
        dest=a0
        log.debug("setting %s selector to %d" % (self.Selectors[sender][0], dest))
        self.hw.setDiscrete(self.Selectors[sender][0], dest)

    # public slots: mix output controls
    def set_mix1_dest(self,a0):
        self.setMixDest('mix1', a0)

    def setSelector(self,a0,a1):
        name=a0
        state = a1
        log.debug("setting %s state to %d" % (name, state))
        self.hw.setDiscrete(self.SelectorControls[name][0], state)

    # Hide and disable a control
    def disable_hide(self,widget):
        widget.hide()
        widget.setEnabled(False)

    # Set destination lists for the 8pre.  MainOut is index 2 and for the
    # moment we assume the ADAT options follow that.
    def set_8pre_dest_list(self, list):
        list.setItemText(2, "MainOut")
        list.setItemText(3, "ADAT 1-2")
        list.setItemText(4, "ADAT 3-4")
        list.setItemText(5, "ADAT 5-6")
        list.setItemText(6, "ADAT 7-8")
        list.setMaxCount(7)

    def initValues_g1(self):
        # Set up widgets for generation-1 devices (only the 828mk1 for now). 
        # For now disable all mix faders and analog controls since we don't
        # know how to control them (and it's not clear that they are even
        # present in the hardware).
        self.disable_hide(self.mixtab)
        self.disable_hide(self.masterbox)
        self.disable_hide(self.analog_settings_box)

        # The 828mk1 didn't have meter controls
        self.disable_hide(self.meter_src)
        self.disable_hide(self.aesebu_meter)
        self.disable_hide(self.peakhold_time)
        self.disable_hide(self.cliphold_time)

        # For the moment hide this control too since it's not clear how
        # it might be used to interact with the 828mk1.  Ultimately we
        # may be able to reset its items and use it for the monitor source
        # selector.
        self.disable_hide(self.phones_src)
        self.disable_hide(self.phones_src_frame)

    def initValues_g2(self):
        # Set up widgets for generation-2 devices

        # The 828Mk2 has separate Mic inputs but no AES/EBU, so use the
        # AES/EBU mixer controls as "Mic" controls.  If a device comes along
        # with both mic and AES inputs this approach will have to be
        # re-thought.
        # Doing this means that on the 828Mk2, the mixer matrix elements
        # used for AES/EBU on other models are used for the Mic channels. 
        # So long as the MixerChannels_828Mk2 definition in
        # motu_avdevice.cpp defines the mic channels immediately after the 8
        # analog channels we'll be right.  Note that we don't need to change
        # the matrix lookup tables (self.ChannelFaders etc) because the QT
        # controls are still named *aesebu*.
        if (self.model == MOTU_MODEL_828mkII):
            self.mix1_tab.setTabText(1, "Mic inputs");
            self.mix2_tab.setTabText(1, "Mic inputs");
            self.mix3_tab.setTabText(1, "Mic inputs");
            self.mix4_tab.setTabText(1, "Mic inputs");
        else:
            # Only the Traveler and 896HD have AES/EBU inputs, so disable the AES/EBU
            # tab for all other models.
            if (self.model!=MOTU_MODEL_TRAVELER and self.model!=MOTU_MODEL_896HD):
                self.mix1_tab.setTabEnabled(1, False)
                self.mix2_tab.setTabEnabled(1, False)
                self.mix3_tab.setTabEnabled(1, False)
                self.mix4_tab.setTabEnabled(1, False)

        # All models except the 896HD and 8pre have SPDIF inputs.
        if (self.model==MOTU_MODEL_8PRE or self.model==MOTU_MODEL_896HD):
            self.mix1_tab.setTabEnabled(2, False);
            self.mix2_tab.setTabEnabled(2, False);
            self.mix3_tab.setTabEnabled(2, False);
            self.mix4_tab.setTabEnabled(2, False);

        # Devices without AES/EBU inputs/outputs (currently all except the
        # Traveler and 896HD) have dedicated "MainOut" outputs instead. 
        # AES/EBU is normally ID 6 in the destination lists and "MainOut"
        # displaces it on non-AES/EBU models.  The 896HD has both AES/EBU
        # and MainOut which complicates this; it uses ID 6 for MainOut and
        # ID 7 (nominally SPDIF) for AES/EBU.  Therefore change ID 6 to
        # "MainOut" for everything but the Traveler, and set ID 7 (nominally
        # SPDIF) to AES/EBU for the 896HD.
        if (self.model != MOTU_MODEL_TRAVELER):
            self.mix1_dest.setItemText(6, "MainOut")
            self.mix2_dest.setItemText(6, "MainOut")
            self.mix3_dest.setItemText(6, "MainOut")
            self.mix4_dest.setItemText(6, "MainOut")
            self.phones_src.setItemText(6, "MainOut")
        if (self.model == MOTU_MODEL_896HD):
            self.mix1_dest.setItemText(7, "AES/EBU")
            self.mix2_dest.setItemText(7, "AES/EBU")
            self.mix3_dest.setItemText(7, "AES/EBU")
            self.mix4_dest.setItemText(7, "AES/EBU")
            self.phones_src.setItemText(7, "AES/EBU")

        # The Ultralite doesn't have ADAT channels (or any optical ports at
        # all)
        if (self.model == MOTU_MODEL_ULTRALITE):
            self.mix1_tab.setTabEnabled(3, False)  # ADAT page
            self.mix2_tab.setTabEnabled(3, False)  # ADAT page
            self.mix3_tab.setTabEnabled(3, False)  # ADAT page
            self.mix4_tab.setTabEnabled(3, False)  # ADAT page
            self.optical_in_mode.setEnabled(False)
            self.optical_out_mode.setEnabled(False)

        # The 896HD and 8pre don't have optical SPDIF (aka Toslink) capability
        if (self.model==MOTU_MODEL_896HD or self.model==MOTU_MODEL_8PRE):
            self.optical_in_mode.removeItem(2)
            self.optical_out_mode.removeItem(2)

        # The 8pre doesn't have software phones/main fader controls
        if (self.model==MOTU_MODEL_8PRE):
            self.disable_hide(self.mainout_fader)
            self.disable_hide(self.phones_fader)
            self.disable_hide(self.masterbox)

        # The 8pre's destination list is rather different to all other 
        # models.
        if (self.model==MOTU_MODEL_8PRE):
            self.set_8pre_dest_list(self.mix1_dest)
            self.set_8pre_dest_list(self.mix2_dest)
            self.set_8pre_dest_list(self.mix3_dest)
            self.set_8pre_dest_list(self.mix4_dest)
            self.set_8pre_dest_list(self.phones_src)

        # Only the 896HD has meter controls
        if (self.model != MOTU_MODEL_896HD):
            self.disable_hide(self.meter_src)
            self.disable_hide(self.aesebu_meter)
            self.disable_hide(self.peakhold_time)
            self.disable_hide(self.cliphold_time)

        # Some controls must be disabled if the device is streaming
        if (self.is_streaming):
            log.debug("Disabling controls which require inactive streaming")
            self.optical_in_mode.setEnabled(False)
            self.optical_out_mode.setEnabled(False)

        # Some channels aren't available at higher sampling rates
        if (self.sample_rate > 96000):
            log.debug("Disabling controls not present above 96 kHz")
            self.mix1_tab.setTabEnabled(3, False)  # ADAT
            self.mix1_tab.setTabEnabled(2, False)  # SPDIF
            self.mix1_tab.setTabEnabled(1, False)  # AES/EBU
            self.mix2_tab.setTabEnabled(3, False)  # ADAT
            self.mix2_tab.setTabEnabled(2, False)  # SPDIF
            self.mix2_tab.setTabEnabled(1, False)  # AES/EBU
            self.mix3_tab.setTabEnabled(3, False)  # ADAT
            self.mix3_tab.setTabEnabled(2, False)  # SPDIF
            self.mix3_tab.setTabEnabled(1, False)  # AES/EBU
            self.mix4_tab.setTabEnabled(3, False)  # ADAT
            self.mix4_tab.setTabEnabled(2, False)  # SPDIF
            self.mix4_tab.setTabEnabled(1, False)  # AES/EBU
        if (self.sample_rate > 48000):
            log.debug("Disabling controls not present above 48 kHz")
            self.mix1adat5.setEnabled(False)
            self.mix1adat6.setEnabled(False)
            self.mix1adat7.setEnabled(False)
            self.mix1adat8.setEnabled(False)
            self.mix2adat5.setEnabled(False)
            self.mix2adat6.setEnabled(False)
            self.mix2adat7.setEnabled(False)
            self.mix2adat8.setEnabled(False)
            self.mix3adat5.setEnabled(False)
            self.mix3adat6.setEnabled(False)
            self.mix3adat7.setEnabled(False)
            self.mix3adat8.setEnabled(False)
            self.mix4adat5.setEnabled(False)
            self.mix4adat6.setEnabled(False)
            self.mix4adat7.setEnabled(False)
            self.mix4adat8.setEnabled(False)

        # Ensure the correct input controls are active for a given interface.
        # Only the Ultralite has phase inversion switches.
        if (not(self.model == MOTU_MODEL_ULTRALITE)):
            self.disable_hide(self.ana1_invert)
            self.disable_hide(self.ana2_invert)
            self.disable_hide(self.ana3_invert)
            self.disable_hide(self.ana4_invert)
            self.disable_hide(self.ana5_invert)
            self.disable_hide(self.ana6_invert)
            self.disable_hide(self.ana7_invert)
            self.disable_hide(self.ana8_invert)
            self.disable_hide(self.spdif1_invert)
            self.disable_hide(self.spdif2_invert)
        # The Traveler has pad switches for analog 1-4 only; other interfaces
        # don't have pad switches at all.
        if (not(self.model == MOTU_MODEL_TRAVELER)):
            self.disable_hide(self.ana1_pad)
            self.disable_hide(self.ana2_pad)
            self.disable_hide(self.ana3_pad)
            self.disable_hide(self.ana4_pad)
        self.disable_hide(self.ana5_pad)
        self.disable_hide(self.ana6_pad)
        self.disable_hide(self.ana7_pad)
        self.disable_hide(self.ana8_pad)
        # The Traveler has level and boost switches for analog 5-8.  The
        # 8pre, Ultralite and the 896HD don't implement them.  All other
        # interfaces have them over analog 1-8.
        if (self.model==MOTU_MODEL_TRAVELER or self.model==MOTU_MODEL_ULTRALITE or self.model==MOTU_MODEL_896HD or self.model==MOTU_MODEL_8PRE):
            self.disable_hide(self.ana1_level)
            self.disable_hide(self.ana2_level)
            self.disable_hide(self.ana3_level)
            self.disable_hide(self.ana4_level)
            self.disable_hide(self.ana1_boost)
            self.disable_hide(self.ana2_boost)
            self.disable_hide(self.ana3_boost)
            self.disable_hide(self.ana4_boost)
        if (self.model==MOTU_MODEL_ULTRALITE or self.model==MOTU_MODEL_896HD or self.model==MOTU_MODEL_8PRE):
            self.disable_hide(self.ana5_level)
            self.disable_hide(self.ana6_level)
            self.disable_hide(self.ana7_level)
            self.disable_hide(self.ana8_level)
            self.disable_hide(self.ana5_boost)
            self.disable_hide(self.ana6_boost)
            self.disable_hide(self.ana7_boost)
            self.disable_hide(self.ana8_boost)
        # The Traveler has trimgain for analog 1-4.  The Ultralite has trimgain for
        # analog 1-8 and SPDIF 1-2.  All other interfaces don't have trimgain.
        if (not(self.model==MOTU_MODEL_TRAVELER or self.model==MOTU_MODEL_ULTRALITE)):
            self.disable_hide(self.ana1_trimgain)
            self.disable_hide(self.ana1_trimgain_label)
            self.disable_hide(self.ana2_trimgain)
            self.disable_hide(self.ana2_trimgain_label)
            self.disable_hide(self.ana3_trimgain)
            self.disable_hide(self.ana3_trimgain_label)
            self.disable_hide(self.ana4_trimgain)
            self.disable_hide(self.ana4_trimgain_label)
        if (not(self.model == MOTU_MODEL_ULTRALITE)):
            self.disable_hide(self.ana5_trimgain)
            self.disable_hide(self.ana5_trimgain_label)
            self.disable_hide(self.ana6_trimgain)
            self.disable_hide(self.ana6_trimgain_label)
            self.disable_hide(self.ana7_trimgain)
            self.disable_hide(self.ana7_trimgain_label)
            self.disable_hide(self.ana8_trimgain)
            self.disable_hide(self.ana8_trimgain_label)
            self.disable_hide(self.spdif1_trimgain)
            self.disable_hide(self.spdif1_trimgain_label)
            self.disable_hide(self.spdif1ctrl)
            self.disable_hide(self.spdif2_trimgain)
            self.disable_hide(self.spdif2_trimgain_label)
            self.disable_hide(self.spdif2ctrl)

        # The 8pre has no digital controls for the analog inputs, so there's
        # no point in showing the frame which would normally contain them.
        if (self.model == MOTU_MODEL_8PRE):
            self.disable_hide(self.analog_settings_box)

    def initValues(self):
        # Is the device streaming?
        self.is_streaming = self.hw.getDiscrete('/Mixer/Info/IsStreaming')
        log.debug("device streaming flag: %d" % (self.is_streaming))

        # Retrieve other device settings as needed and customise the UI
        # based on these options.
        self.model = self.hw.getDiscrete('/Mixer/Info/Model')
        log.debug("device model identifier: %d" % (self.model))
        self.sample_rate = self.hw.getDiscrete('/Mixer/Info/SampleRate')
        log.debug("device sample rate: %d" % (self.sample_rate))

        # MOTU Mark 3 devices should be configured to use the "Motu_Mark3" 
        # mixer.
        if (self.model==MOTU_MODEL_828mk3 or self.model==MOTU_MODEL_ULTRALITEmk3 or self.model==MOTU_MODEL_ULTRALITEmk3_HYB or self.model==MOTU_MODEL_TRAVELERmk3 or self.model==MOTU_MODEL_896HDmk3):
            log.debug("Generation-3 MOTU devices should be configured to use the Motu_Mark3 mixer")
            return
        elif (self.model==MOTU_MODEL_828MkI):
            self.initValues_g1()
        else:
            self.initValues_g2()

        # Now fetch the current values into the respective controls.  Don't
        # bother fetching controls which are disabled.
        for ctrl, info in self.ChannelFaders.items():
            if (not(ctrl.isEnabled())):
                continue
            vol = self.hw.getMatrixMixerValue(info[0], info[1], info[2])
            log.debug("%s for mix %d channel %d is %d" % (info[0], info[1], info[2], vol))
            ctrl.setValue(vol)
            ctrl.valueChanged.connect(self.updateChannelFader)

        for ctrl, info in self.Faders.items():
            if (not(ctrl.isEnabled())):
                continue
            vol = self.hw.getDiscrete(info[0])
            log.debug("%s mix fader is %d" % (info[0] , vol))
            ctrl.setValue(vol)
            ctrl.valueChanged.connect(self.updateFader)

        for ctrl, info in self.ChannelControls.items():
            if (not(ctrl.isEnabled())):
                continue
            pan = self.hw.getMatrixMixerValue(info[0], info[1], info[2])
            log.debug("%s for mix %d channel %d is %d" % (info[0], info[1], info[2], pan))
            ctrl.setValue(pan)
            ctrl.valueChanged.connect(self.updateChannelControl)

        for ctrl, info in self.Controls.items():
            if (not(ctrl.isEnabled())):
                continue
            pan = self.hw.getDiscrete(info[0])
            log.debug("%s control is %d" % (info[0] , pan))
            ctrl.setValue(pan)
            ctrl.valueChanged.connect(self.updateControl)

        for ctrl, info in self.ChannelBinarySwitches.items():
            if (not(ctrl.isEnabled())):
                continue
            val = self.hw.getMatrixMixerValue(info[0], info[1], info[2])
            log.debug("%s for mix %d channel %d is %d" % (info[0] , info[1], info[2], val))
            if val:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)
            ctrl.toggled.connect(self.updateChannelBinarySwitch)

        for ctrl, info in self.BinarySwitches.items():
            if (not(ctrl.isEnabled())):
                continue
            val = self.hw.getDiscrete(info[0])
            log.debug("%s switch is %d" % (info[0] , val))
            if val:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)
            ctrl.toggled.connect(self.updateBinarySwitch)

        for ctrl, info in self.Selectors.items():
            if (not(ctrl.isEnabled())):
                continue
            dest = self.hw.getDiscrete(info[0])
            log.debug("%s selector is %d" % (info[0] , dest))
            ctrl.setCurrentIndex(dest)
            ctrl.activated.connect(self.updateSelector)

        # We could enable/disable ADAT controls here depending on whether
        # the optical port is set to ADAT or something else.  A disable
        # can't be done earlier since we have to read the ADAT mixer
        # settings (which won't happen if they're disabled).  However, on
        # the other hand it may be more convenient to leave all controls
        # active at all times.  We'll see.

# vim: et



#
# Copyright (C) 2009, 2011 by Jonathan Woithe
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4 import QtGui
# from PyQt4.QtCore import QObject, Qt, QTimer
# from PyQt4.QtGui import QWidget, QApplication, QVBoxLayout
from ffado.import_pyqt import *

from ffado.config import *

from ffado.widgets.matrixmixer import MatrixMixer

import logging
log = logging.getLogger('rme')

# Model defines.  These must agree with what is used in rme_avdevice.h.
RME_MODEL_NONE      = 0x0000
RME_MODEL_FF800     = 0x0001
RME_MODEL_FF400     = 0x0002

class Rme(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/rme", self)

        self.init()

    def init(self):

        self.PhantomSwitches={
            self.phantom_0: ['/Control/Phantom', 0],
            self.phantom_1: ['/Control/Phantom', 1],
            self.phantom_2: ['/Control/Phantom', 2],
            self.phantom_3: ['/Control/Phantom', 3],
        }

        self.Switches={
            self.ff400_chan3_opt_instr: ['/Control/Chan3_opt_instr'],
            self.ff400_chan3_opt_pad:   ['/Control/Chan3_opt_pad'],
            self.ff400_chan4_opt_instr: ['/Control/Chan4_opt_instr'],
            self.ff400_chan4_opt_pad:   ['/Control/Chan4_opt_pad'],

            self.spdif_output_optical:  ['/Control/SPDIF_output_optical', 0],
            self.spdif_output_emphasis: ['/Control/SPDIF_output_emphasis', 0],
            self.spdif_output_pro:      ['/Control/SPDIF_output_pro', 0],
            self.spdif_output_nonaudio: ['/Control/SPDIF_output_nonaudio', 0],
        }

        self.Radiobuttons={
            self.level_in_lo_gain: ['/Control/Input_level', 0],
            self.level_in_p4dBu:   ['/Control/Input_level', 2],
            self.level_in_m10dBV:  ['/Control/Input_level', 1],

            self.level_out_hi_gain: ['/Control/Output_level', 2],
            self.level_out_p4dBu:   ['/Control/Output_level', 1],
            self.level_out_m10dBV:  ['/Control/Output_level', 0],

            self.spdif_input_coax:    ['/Control/SPDIF_input_mode', 0],
            self.spdif_input_optical: ['/Control/SPDIF_input_mode', 1],

            self.phones_hi_gain: ['/Control/Phones_level', 0],
            self.phones_p4dBu:   ['/Control/Phones_level', 1],
            self.phones_m10dBV:  ['/Control/Phones_level', 2],

            self.clock_mode_autosync: ['/Control/Clock_mode', 1],
            self.clock_mode_master: ['/Control/Clock_mode', 0],

            self.sync_ref_wordclk: ['/Control/Sync_ref', 0],
            self.sync_ref_adat1: ['/Control/Sync_ref', 1],
            self.sync_ref_adat2: ['/Control/Sync_ref', 2],
            self.sync_ref_spdif: ['/Control/Sync_ref', 3],
            self.sync_ref_tco: ['/Control/Sync_ref', 4],
            
            # TCO controls
            self.sync_source_ltc: ['/Control/Tco_sync_src', 0],
            self.sync_source_video: ['/Control/Tco_sync_src', 1],
            self.sync_source_wordclk: ['/Control/Tco_sync_src', 2],
            self.frame_rate_24fps: ['/Control/Tco_frame_rate', 0],
            self.frame_rate_25fps: ['/Control/Tco_frame_rate', 1],
            self.frame_rate_29_97fps: ['/Control/Tco_frame_rate', 2],
            self.frame_rate_29_97dfps: ['/Control/Tco_frame_rate', 3],
            self.frame_rate_30fps: ['/Control/Tco_frame_rate', 4],
            self.frame_rate_30dfps: ['/Control/Tco_frame_rate', 5],
            self.sample_rate_44k1: ['/Control/Tco_sample_rate', 0],
            self.sample_rate_48k: ['/Control/Tco_sample_rate', 1],
            self.sample_rate_0pc: ['/Control/Tco_sample_rate_ofs', 0],
            self.sample_rate_p01pc: ['/Control/Tco_sample_rate_ofs', 1],
            self.sample_rate_n01pc: ['/Control/Tco_sample_rate_ofs', 2],
            self.sample_rate_p40pc: ['/Control/Tco_sample_rate_ofs', 3],
            self.sample_rate_n40pc: ['/Control/Tco_sample_rate_ofs', 4],
            self.wordclk_conv_1_1: ['/Control/Tco_word_clk_conv', 0],
            self.wordclk_conv_44k1_48k: ['/Control/Tco_word_clk_conv', 1],
            self.wordclk_conv_48k_44k1: ['/Control/Tco_word_clk_conv', 2],
        }

        self.Checkboxes={
            self.ch1_instr_fuzz: ['/Control/Chan1_instr_opts', 0x04],
            self.ch1_instr_limiter: ['/Control/Chan1_instr_opts', 0x08],
            self.ch1_instr_filter: ['/Control/Chan1_instr_opts', 0x02],

            # TCO controls
            self.video_input_termination_on: ['/Control/Tco_video_in_term', 0x01],
        }

        self.Gains={
            self.gain_mic1: ['/Control/Gains', 0],
            self.gain_mic2: ['/Control/Gains', 1],
            self.gain_input3: ['/Control/Gains', 2],
            self.gain_input4: ['/Control/Gains', 3],
        }

        self.Combos={
            self.ff800_ch1_src: ['/Control/Chan1_source'],
            self.ff800_ch7_src: ['/Control/Chan7_source'],
            self.ff800_ch8_src: ['/Control/Chan8_source'],
        }

        self.CommandButtons={
            self.control_load: ['/Control/Flash_control', 0],
            self.control_save: ['/Control/Flash_control', 1],
            self.mixer_load:   ['/Control/Flash_control', 2],
            self.mixer_save:   ['/Control/Flash_control', 3],
            self.mixer_preset_ffado_default: ['/Control/Mixer_preset', 0],
        }

        # Other mixer variables
        self.is_streaming = 0
        self.sample_rate = 0
        self.model = 0
        self.tco_present = 0

    # Public slot: update phantom power hardware switches
    def updatePhantomSwitch(self, a0):
        sender = self.sender()
        # Value is the phantom switch value, with a corresponding enable
        # bit in the high 16 bit word
        val = (a0 << self.PhantomSwitches[sender][1]) | (0x00010000 << self.PhantomSwitches[sender][1])
        log.debug("phantom switch %d set to %d" % (self.PhantomSwitches[sender][1], a0))
        self.hw.setDiscrete(self.PhantomSwitches[sender][0], val)

    # Public slot: update generic switches
    def updateSwitch(self, a0):
        sender = self.sender()
        log.debug("switch %s set to %d" % (self.Switches[sender][0], a0))
        self.hw.setDiscrete(self.Switches[sender][0], a0)

    # Public slot: update generic radiobuttons
    def updateRadiobutton(self, a0):
        sender = self.sender()
        if (a0 != 0):
            # Only change the control state on a button being "checked"
            log.debug("radiobutton group %s set to %d" % (self.Radiobuttons[sender][0], self.Radiobuttons[sender][1]))
            self.hw.setDiscrete(self.Radiobuttons[sender][0], self.Radiobuttons[sender][1])

    def updateCheckboxes(self, a0):
        sender = self.sender()
        val = self.hw.getDiscrete(self.Checkboxes[sender][0]);
        if (a0 != 0):
            val = val | self.Checkboxes[sender][1]
        else:
            val = val & ~self.Checkboxes[sender][1]
        log.debug("checkbox group %s set to %d" % (self.Checkboxes[sender][0], val));
        self.hw.setDiscrete(self.Checkboxes[sender][0], val)

    # Public slot: update gains
    def updateGain(self, a0):
        sender = self.sender()
        log.debug("gain %s[%d] set to %d" % (self.Gains[sender][0], self.Gains[sender][1], a0))
        self.hw.setMatrixMixerValue(self.Gains[sender][0], 0, self.Gains[sender][1], a0)

    def updateBandwidthLimit(self, a0):
        # Account for the "No ADAT-2" item which will not be present on
        # a FF400.
        if (self.model==RME_MODEL_FF400 and a0>0):
            a0 = a0 + 1
        # log.debug("limit update: %d" % (a0));
        self.hw.setDiscrete('/Control/Bandwidth_limit', a0);

    # Public slot: send command
    def sendCommand(self, a0):
        sender = self.sender()
        log.debug("command %d sent to %s" % (self.CommandButtons[sender][1], self.CommandButtons[sender][0]))
        self.hw.setDiscrete(self.CommandButtons[sender][0], self.CommandButtons[sender][1])

        # If mixer values have been reloaded, refresh the mixer GUI.  This
        # will also commit the new values to the hardware via the "changed"
        # signal handlers of the mixer elements.
        if (self.CommandButtons[sender][1] == 2):
            self.inputmatrix.refreshValues()
            self.outputmatrix.refreshValues()
            self.playbackmatrix.refreshValues()

        # If settings have been reloaded from flash, refresh the GUI.  The
        # settings will be made active in the hardware via the "changed"
        # signal handlers of the respective GUI control widgets.
        if (self.CommandButtons[sender][1] == 0):
            self.getValuesFromFF()

    def updateCombo(self, a0):
        sender = self.sender()
        log.debug("combo %s set to %d" % (self.Combos[sender][0], a0))
        self.hw.setDiscrete(self.Combos[sender][0], a0)

        # Enable the limiter control only when the front source is active
        if (sender == self.ff800_ch1_src):
            self.ch1_instr_limiter.setEnabled(a0==0)

    def updateStreamingState(self):
        ss = self.streamingstatus.selected()
        ss_txt = self.streamingstatus.getEnumLabel(ss)
        if ss_txt != 'Idle':
            self.is_streaming = True
        else:
            self.is_streaming = False
        if (self.last_streaming_state != self.is_streaming):
            self.bandwidth_limit.setEnabled(not(self.is_streaming));
            self.control_load.setEnabled(not(self.is_streaming));
        self.last_streaming_state = self.is_streaming

    def status_update(self):
        # log.debug("timer event")
        self.updateStreamingState()
        clk_mode = ['Master', 'Slave']
        src_str = ['None', 'ADAT 1', 'ADAT 2', 'SPDIF', 'Wordclock', 'TCO']
        sync_stat = ['No lock', 'Locked', 'Synced']
        sysclock_mode = self.hw.getDiscrete('/Control/sysclock_mode')
        sysclock_freq = self.hw.getDiscrete('/Control/sysclock_freq')
        autosync_freq = self.hw.getDiscrete('/Control/autosync_freq')
        autosync_src = self.hw.getDiscrete('/Control/autosync_src')
        sync_status = self.hw.getDiscrete('/Control/sync_status')
        spdif_freq = self.hw.getDiscrete('/Control/spdif_freq')
        self.sysclock_freq.setText("%d Hz" % (sysclock_freq))
        self.sysclock_mode.setText(clk_mode[sysclock_mode])
        self.autosync_freq.setText("%d Hz" % (autosync_freq))
        self.autosync_src.setText(src_str[autosync_src])
        self.sync_check_adat1_status.setText(sync_stat[sync_status & 0x03])
        self.sync_check_adat2_status.setText(sync_stat[(sync_status >> 2) & 0x03])
        self.sync_check_spdif_status.setText(sync_stat[(sync_status >> 4) & 0x03])
        self.sync_check_wclk_status.setText(sync_stat[(sync_status >> 6) & 0x03])
        self.sync_check_tco_status.setText(sync_stat[(sync_status >> 8) & 0x03])
        self.spdif_freq.setText("%d Hz" % (spdif_freq))
        
        if (self.tco_present):
            ltc_valid_str = ['Not detected', 'Valid']
            ltc_framerate_str = ['24 fps', '25 fps', '29.97 fps', '30 fps']
            ltc_frametype_str = ['Normal (full frame)', 'Dropframe']
            video_type_str = ['No video', 'PAL', 'NTSC']
            word_clock_str = ['None', 'Single Speed', 'Double Speed', 'Quad Speed']
            status_str = ['Not locked', "Locked"]
            ltc = self.hw.getDiscrete('/Control/Tco_ltc_in')
            ltc_valid = self.hw.getDiscrete('/Control/Tco_input_ltc_valid')
            ltc_fps = self.hw.getDiscrete('/Control/Tco_input_ltc_fps')
            ltc_dropframe = self.hw.getDiscrete('/Control/Tco_input_ltc_dropframe')
            videotype = self.hw.getDiscrete('/Control/Tco_input_video_type')
            wordclk = self.hw.getDiscrete('/Control/Tco_input_word_clk')
            input_lock = self.hw.getDiscrete('/Control/Tco_input_lock')
            tco_freq = self.hw.getDiscrete('/Control/Tco_freq')
            self.state_ltc_valid_label.setText(ltc_valid_str[ltc_valid])
            if (ltc_valid):
                self.ltc_in_hours.setText("%02d" % (ltc >> 24))
                self.ltc_in_minutes.setText("%02d" % ((ltc >> 16) & 0xff))
                self.ltc_in_seconds.setText("%02d" % ((ltc >> 8) & 0xff))
                self.ltc_in_frames.setText("%02d" % (ltc & 0xff))
                self.state_ltc_framerate.setText(ltc_framerate_str[ltc_fps])
                self.state_ltc_frame_type.setText(ltc_frametype_str[ltc_dropframe])
            else:
                self.ltc_in_hours.setText("--")
                self.ltc_in_minutes.setText("--")
                self.ltc_in_seconds.setText("--")
                self.ltc_in_frames.setText("--")
                self.state_ltc_framerate.setText("-")
                self.state_ltc_frame_type.setText("-")
            self.state_video_type.setText(video_type_str[videotype])
            self.state_word_clk.setText(word_clock_str[wordclk])
            self.sync_source_status.setText(status_str[input_lock])
            self.tco_frequency_label.setText("%d Hz" % (tco_freq))

    # Hide and disable a control
    def disable_hide(self,widget):
        for w in widget.children():
            if isinstance(w, QWidget):
                w.hide()
                w.setEnabled(False)
        widget.hide()
        widget.setEnabled(False)

    def setupSignals(self):

        # Connect signal handlers for all command buttons
        for ctrl, info in self.CommandButtons.items():
            if (not(ctrl.isEnabled())):
                continue
            ctrl.clicked.connect(self.sendCommand)

        for ctrl, info in self.Combos.items():
            if (not(ctrl.isEnabled())):
                continue;
            ctrl.currentIndexChanged.connect(self.updateCombo)

        self.bandwidth_limit.currentIndexChanged.connect(self.updateBandwidthLimit)

        # Get current hardware values and connect GUI element signals to 
        # their respective slots
        for ctrl, info in self.PhantomSwitches.items():
            if (not(ctrl.isEnabled())):
                continue
            ctrl.toggled.connect(self.updatePhantomSwitch)

        for ctrl, info in self.Switches.items():
            if (not(ctrl.isEnabled())):
                continue
            ctrl.toggled.connect(self.updateSwitch)

        for ctrl, info in self.Radiobuttons.items():
            if (not(ctrl.isEnabled())):
                continue;
            ctrl.toggled.connect(self.updateRadiobutton)

        for ctrl, info in self.Checkboxes.items():
            if (not(ctrl.isEnabled())):
                continue;
            ctrl.toggled.connect(self.updateCheckboxes)

        for ctrl, info in self.Gains.items():
            if (not(ctrl.isEnabled())):
                continue
            ctrl.valueChanged.connect(self.updateGain)

    # Obtain control values from the Fireface and make the GUI reflect these
    def getValuesFromFF(self):
        for ctrl, info in self.Combos.items():
            if (not(ctrl.isEnabled())):
                continue;
            val = self.hw.getDiscrete(info[0])
            log.debug("combo %s is %d" % (info[0], val));
            ctrl.setCurrentIndex(val);

        # Set the bandwidth limit control to reflect the current device
        # setting, allowing for the additional "No ADAT-2" item which is
        # present on the FF800.
        val = self.hw.getDiscrete('/Control/Bandwidth_limit')
        if (self.model==RME_MODEL_FF400 and val>1):
            val = val - 1
        self.bandwidth_limit.setCurrentIndex(val);

        # Get current hardware values
        for ctrl, info in self.PhantomSwitches.items():
            if (not(ctrl.isEnabled())):
                continue
            val = (self.hw.getDiscrete(info[0]) >> info[1]) & 0x01
            log.debug("phantom switch %d is %d" % (info[1], val))
            if val:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)

        for ctrl, info in self.Switches.items():
            if (not(ctrl.isEnabled())):
                continue
            val = self.hw.getDiscrete(info[0])
            log.debug("switch %s is %d" % (info[0], val))
            if val:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)

        for ctrl, info in self.Radiobuttons.items():
            if (not(ctrl.isEnabled())):
                continue;
            # This is a touch wasteful since it means we retrieve the control
            # value once per radio button rather than once per radio button
            # group.  In time we might introduce radiobutton groupings in the
            # self.* datastructures to avoid this, but for the moment this is
            # easy and it works.
            val = self.hw.getDiscrete(info[0])
            if (val == info[1]):
                val = 1
            else:
                val = 0
            ctrl.setChecked(val)
            log.debug("Radiobutton %s[%d] is %d" % (info[0], info[1], val))

        # Make sure the Limiter control can receive a value
        if (self.ff800_ch1_src.isEnabled()):
            self.ch1_instr_limiter.setEnabled(1)

        for ctrl, info in self.Checkboxes.items():
            if (not(ctrl.isEnabled())):
                continue;
            # This is a touch wasteful since it means we retrieve the control
            # value once per checkbox button rather than once per checkbox
            # group.  In time we might introduce checkbox groupings in the
            # self.* datastructures to avoid this, but for the moment this is
            # easy and it works.
            val = self.hw.getDiscrete(info[0])
            if (val & info[1]):
                val = 1
            else:
                val = 0
            ctrl.setChecked(val)
            log.debug("Checkbox %s[%d] is %d" % (info[0], info[1], val))

        # The limiter (a checkbox) can only be controlled if the front
        # source is selected for channel 1.
        ch1_src = self.ff800_ch1_src.currentIndex()
        if (self.ff800_ch1_src.isEnabled()):
            self.ch1_instr_limiter.setEnabled(ch1_src==0)

        for ctrl, info in self.Gains.items():
            if (not(ctrl.isEnabled())):
                continue
            val = self.hw.getMatrixMixerValue(info[0], 0, info[1])
            log.debug("gain %s[%d] is %d" % (info[0], info[1], val))
            ctrl.setValue(val);



    def initValues(self):

        # print( self.hw.servername )
        # print( self.hw.basepath )
        self.inputmatrix = MatrixMixer(self.hw.servername, self.hw.basepath+"/Mixer/InputFaders", self, "Columns_are_inputs", 0x8000, self.hw.basepath+"/Mixer/InputMutes", self.hw.basepath+"/Mixer/InputInverts", True)
        layout = QVBoxLayout()
        layout.addWidget(self.inputmatrix)
        self.mixer.setLayout(layout)

        self.playbackmatrix = MatrixMixer(self.hw.servername, self.hw.basepath+"/Mixer/PlaybackFaders", self, "Columns_are_inputs", 0x8000, self.hw.basepath+"/Mixer/PlaybackMutes", self.hw.basepath+"/Mixer/PlaybackInverts", True)
        layout = QVBoxLayout()
        layout.addWidget(self.playbackmatrix)
        self.playbackmixer.setLayout(layout)

        self.outputmatrix = MatrixMixer(self.hw.servername, self.hw.basepath+"/Mixer/OutputFaders", self, "Columns_are_inputs", 0x8000, self.hw.basepath+"/Mixer/OutputMutes", None, True)
        layout = QVBoxLayout()

        # This is a bit of a hack, but it works to ensure this single-row
        # matrix mixer doesn't fill the entire screen but also doesn't end
        # up with a pointless scrollbar.  The matrix mixer's minimum height
        # is 0 according to minimumHeight(), which is probably the
        # fundamental issue here; however, I've already wasted too much time
        # trying to get this to work so if the hack is effective we'll run
        # with it.
        self.outputmatrix.setMinimumHeight(150)
        layout.addWidget(self.outputmatrix, 0, Qt.AlignTop)
        self.outputmixer.setLayout(layout)

        self.is_streaming = False
        self.last_streaming_state = False

        # Disable the "load settings" button if streaming is active.  Its
        # enable state will be kept up to date by updateStreamingState().
        self.control_load.setEnabled(not(self.is_streaming))

        # Also disable other controls which are not yet implemented.
        self.mixer_preset_ffado_default.setEnabled(False)

        # Retrieve other device settings as needed and customise the UI
        # based on these options.
        self.model = self.hw.getDiscrete('/Control/Model')
        log.debug("device model identifier: %d" % (self.model))
        self.tco_present = self.hw.getDiscrete('/Control/TCO_present')
        log.debug("device has TCO: %d" % (self.tco_present))
        #self.sample_rate = self.hw.getDiscrete('/Mixer/Info/SampleRate')
        #log.debug("device sample rate: %d" % (self.sample_rate))

        # Assume the TCO options tab is the second from the left (index 1)
        if (not(self.tco_present)):
            self.disable_hide(self.tco_options);
            self.tabWidget.setTabEnabled(1, False)
            self.tabWidget.removeTab(1)

        # The Fireface-400 only has 2 phantom-capable channels
        if (self.model == RME_MODEL_FF400):
            self.disable_hide(self.phantom_2)
            self.disable_hide(self.phantom_3)
        else:
            self.phantom_0.setText("Mic 7")
            self.phantom_1.setText("Mic 8")
            self.phantom_2.setText("Mic 9")
            self.phantom_3.setText("Mic 10")

        # Instrument options, input jack selection controls and an ADAT2
        # input are applicable only to the FF800
        if (self.model != RME_MODEL_FF800):
            self.instrument_options_group.setEnabled(False)
            self.input_plug_select_group.setEnabled(False)
            self.sync_ref_adat2.setEnabled(False)
            self.sync_check_adat2_label.setEnabled(False)
            self.sync_check_adat2_status.setEnabled(False)
            self.spdif_output_optical.setText("ADAT optical")
            self.spdif_input_optical.setText("ADAT optical")

        if (not(self.tco_present)):
            self.sync_check_tco_label.setEnabled(False)
            self.sync_check_tco_status.setEnabled(False)
            self.sync_ref_tco.setEnabled(False)

        # Only the FF400 has specific channel 3/4 options, input gain
        # controls and switchable phones level
        if (self.model != RME_MODEL_FF400):
            # Hide the upper-level frame (and everything in it) to ensure it 
            # requests no vertical space when its contents aren't needed.
            self.disable_hide(self.igains_chan34_opts_frame)
            self.phones_level_group.setEnabled(False)

        # Add the "No ADAT-2" item to the bandwidth limit control if the
        # device is not a FF400.
        if (self.model != RME_MODEL_FF400):
            self.bandwidth_limit.insertItem(1, "No ADAT-2")

        self.getValuesFromFF()
        self.setupSignals()

        # Ensure the limiter checkbox has a signal handler associated with
        # it.  If getValuesFromFF() disabled it because the front input was
        # not selected, setupSignals() would not have configured a handler.
        if (not(self.ch1_instr_limiter.isEnabled())):
            self.ch1_instr_limiter.toggled.connect(self.updateCheckboxes)

        self.updateStreamingState()
        #log.debug("device streaming flag: %d" % (self.is_streaming))

        self.update_timer = QTimer(self)
        self.update_timer.timeout.connect(self.status_update)
        self.update_timer.start(1000)

    def saveSettings(self, indent):
        saveString = []
        idt = indent + "  "
        saveString.append('%s<inputmatrix>\n' % indent)
        saveString.extend(self.inputmatrix.saveSettings(idt))
        # Do not forget to mention the adopted rule for matrix columns mixer
        #  This might be useful for future import function
        saveString.append("%s  <col_rule>\n" % indent)
        saveString.append("%s    Columns_are_inputs\n" % indent)
        saveString.append("%s  </col_rule>\n" % indent)
        saveString.append('%s</inputmatrix>\n' % indent)

        saveString.append('%s<playbackmatrix>\n' % indent)
        saveString.extend(self.playbackmatrix.saveSettings(idt))
        # Do not forget to mention the adopted rule for matrix columns mixer
        #  This might be useful for future import function
        saveString.append("%s  <col_rule>\n" % indent)
        saveString.append("%s    Columns_are_inputs\n" % indent)
        saveString.append("%s  </col_rule>\n" % indent)
        saveString.append('%s</playbackmatrix>\n' % indent)

        saveString.append('%s<outputmatrix>\n' % indent)
        saveString.extend(self.outputmatrix.saveSettings(idt))
        # Do not forget to mention the adopted rule for matrix columns mixer
        #  This might be useful for future import function
        saveString.append("%s  <col_rule>\n" % indent)
        saveString.append("%s    Columns_are_inputs\n" % indent)
        saveString.append("%s  </col_rule>\n" % indent)
        saveString.append('%s</outputmatrix>\n' % indent)
        return saveString

    def readSettings(self, readString):
        try:
            idxb = readString.index('<inputmatrix>')
            idxe = readString.index('</inputmatrix>')
        except Exception:
            log.debug("No Input matrix settings found")
            idxb = -1
            idxe = -1
        if idxb >= 0:
            if idxe > idxb + 1:
                stringMixer = []
                for s in readString[idxb+1:idxe]:
                    stringMixer.append(s)
                # When trying to import from a different device, the rule for column interpretation is
                # not necessarily the same
                try:
                    idx = stringMixer.index('<col_rule>')
                except Exception:
                    log.debug('Do not know how to handle column versus input/output')
                    idx = -1
                transpose_coeff = False
                if idx >=0:
                    if stringMixer[idx+1].find("Columns_are_inputs") == -1:
                        log.debug('Transposing the matrix coefficient; you are importing, are not you ?')
                        transpose_coeff = True
                if self.inputmatrix.readSettings(stringMixer, transpose_coeff):
                    log.debug("Input matrix settings modified")
                del stringMixer

        try:
            idxb = readString.index('<playbackmatrix>')
            idxe = readString.index('</playbackmatrix>')
        except Exception:
            log.debug("No Plaback matrix settings found")
            idxb = -1
            idxe = -1
        if idxb >= 0:
            if idxe > idxb + 1:
                stringMixer = []
                for s in readString[idxb+1:idxe]:
                    stringMixer.append(s)
                # When trying to import from a different device, the rule for column interpretation is
                # not necessarily the same
                try:
                    idx = stringMixer.index('<col_rule>')
                except Exception:
                    log.debug('Do not know how to handle column versus input/output')
                    idx = -1
                transpose_coeff = False
                if idx >=0:
                    if stringMixer[idx+1].find("Columns_are_inputs") == -1:
                        log.debug('Transposing the matrix coefficient; you are importing, are not you ?')
                        transpose_coeff = True
                if self.playbackmatrix.readSettings(stringMixer, transpose_coeff):
                    log.debug("Plaback matrix settings modified")
                del stringMixer

        try:
            idxb = readString.index('<outputmatrix>')
            idxe = readString.index('</outputmatrix>')
        except Exception:
            log.debug("No Output matrix settings found")
            idxb = -1
            idxe = -1
        if idxb >= 0:
            if idxe > idxb + 1:
                stringMixer = []
                for s in readString[idxb+1:idxe]:
                    stringMixer.append(s)
                # When trying to import from a different device, the rule for column interpretation is
                # not necessarily the same
                try:
                    idx = stringMixer.index('<col_rule>')
                except Exception:
                    log.debug('Do not know how to handle column versus input/output')
                    idx = -1
                transpose_coeff = False
                if idx >=0:
                    if stringMixer[idx+1].find("Columns_are_inputs") == -1:
                        log.debug('Transposing the matrix coefficient; you are importing, are not you ?')
                        transpose_coeff = True
                if self.outputmatrix.readSettings(stringMixer, transpose_coeff):
                    log.debug("Output matrix settings modified")
                del stringMixer

# vim: et



#
# Copyright (C) 2005-2008 by Pieter Palmers
# Copyright (C) 2008, 2013 by Jonathan Woithe
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget, QApplication
from ffado.import_pyqt import *

from ffado.config import *

import logging
log = logging.getLogger('motu_mark3')

# Model defines.  These must agree with what is used in motu_avdevice.h.
MOTU_MODEL_NONE             = 0x0000
MOTU_MODEL_828mkII          = 0x0001
MOTU_MODEL_TRAVELER         = 0x0002
MOTU_MODEL_ULTRALITE        = 0x0003
MOTU_MODEL_8PRE             = 0x0004
MOTU_MODEL_828MkI           = 0x0005
MOTU_MODEL_896HD            = 0x0006
MOTU_MODEL_828mk3           = 0x0007
MOTU_MODEL_ULTRALITEmk3     = 0x0008
MOTU_MODEL_ULTRALITEmk3_HYB = 0x0009
MOTU_MODEL_TRAVELERmk3      = 0x000a
MOTU_MODEL_896HDmk3         = 0x000b

class Motu_Mark3(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/motu_mark3", self)

        self.init()

    def init(self):
        # Initialise any object members as needed

        # Other mixer variables
        self.is_streaming = 0
        self.sample_rate = 0
        self.model = MOTU_MODEL_NONE

    # Hide and disable a control
    def disable_hide(self,widget):
        widget.hide()
        widget.setEnabled(False)

    def initValues_g3(self):
        # Set up widgets for generation-3 (aka Mark-3) devices
        return

    def initValues(self):
        # Is the device streaming?
        self.is_streaming = self.hw.getDiscrete('/Mixer/Info/IsStreaming')
        log.debug("device streaming flag: %d" % (self.is_streaming))

        # Retrieve other device settings as needed and customise the UI
        # based on these options.
        self.model = self.hw.getDiscrete('/Mixer/Info/Model')
        log.debug("device model identifier: %d" % (self.model))
        self.sample_rate = self.hw.getDiscrete('/Mixer/Info/SampleRate')
        log.debug("device sample rate: %d" % (self.sample_rate))

        self.initValues_g3()



#
# presonus_inspire1394.py - Qt4/FFADO application for Presonus Inspire1394
# Copyright (C) 2014 Takashi Sakamoto <o-takashi@sakamocchi.jp>
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4 import QtGui, QtCore
# from PyQt4.QtCore import QObject, Qt
# from PyQt4.QtGui import QWidget, QHBoxLayout, QVBoxLayout, QGridLayout
# from PyQt4.QtGui import QGroupBox, QLabel, QSizePolicy, QSlider, QDial, QComboBox, QToolButton
from ffado.import_pyqt import *

from math import log10
from ffado.config import *

class Presonus_Inspire1394(QWidget):
    # feature_id/name
    inputs = [[1, "Analog in 1/2"],
              [2, "Analog in 3/4"]]

    # feature_id/name
    mixer_src = [[3, "Analog in 1/2"],
                 [4, "Analog in 3/4"],
                 [5, "Stream in 1/2"]]

    # feature id/name
    outputs = [[6, "Analog out 1/2"],
               [7, "HP out 1/2"]]

    # selector_id/sources
    out_src = [1, ["Mixer out 1/2", "Stream in 1/2"]]

    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

    def getDisplayTitle(self):
        return 'Inspire1394'

    def buildMixer(self):
        self.Selectors = {}
        self.Volumes = {}
        self.Preamps = {}
        self.Pannings = {}
        self.Mutes = {}

        plain_layout = QHBoxLayout(self)

        left = QGroupBox(self)
        plain_layout.addWidget(left)
        self.addAnalogInputs(left)

        center = QGroupBox(self)
        plain_layout.addWidget(center)
        self.addInternalMixer(center)

        right = QGroupBox(self)
        plain_layout.addWidget(right)
        self.addAnalogOutputs(right)

    def addAnalogInputs(self, box):
        box_layout = QVBoxLayout()
        box.setLayout(box_layout)
        box.setTitle("Analog Inputs")

        grid_layout = QGridLayout()
        box_layout.addLayout(grid_layout)
    
        self.addVolumes(self.inputs, 0, box, grid_layout)

    def addAnalogOutputs(self, box):
        box_layout = QVBoxLayout()
        box.setLayout(box_layout)
        box.setTitle("Analog Outputs")

        grid_layout = QGridLayout()
        box_layout.addLayout(grid_layout)

        self.addVolumes(self.outputs, 2, box, grid_layout)

    def addInternalMixer(self, box):
        box_layout = QGridLayout()
        box.setLayout(box_layout)
        box.setTitle("Hardware Mixer")

        self.addVolumes(self.mixer_src, 1, box, box_layout)

    def addVolumes(self, elms, kind, parent, layout):
        def addPreampParam(label, ch, path, layout):
            button = self.getThinButton(parent, label)
            layout.addWidget(button)
            self.Preamps[button] = ["/Preamp/%s" % path, ch]
            return

        for col in range(len(elms)):
            label = QLabel(parent)
            label.setText(elms[col][1])
            layout.addWidget(label, 0, col * 2, 1, 2, Qt.AlignHCenter | Qt.AlignTop)

            if kind == 0:
                if col == 0:
                    for ch in range(2):
                        box_layout = QVBoxLayout()
                        layout.addLayout(box_layout, 1, col * 2 + ch, Qt.AlignHCenter | Qt.AlignBottom)
                        if col == 0:
                            addPreampParam("+48V", ch + 1, "PhantomPower", box_layout)
                            addPreampParam("Boost", ch + 1, "MicBoost", box_layout)
                            addPreampParam("Limit", ch + 1, "MicLimit", box_layout)
                else:
                    box_layout = QVBoxLayout()
                    addPreampParam("Phono", 0, "PhonoSwitch", box_layout)
                    layout.addLayout(box_layout, 1, col * 2, 1, 2, Qt.AlignHCenter | Qt.AlignBottom)
            elif kind == 1:
                l_dial = self.getDial(parent)
                r_dial = self.getDial(parent)

                layout.addWidget(l_dial, 1, col * 2, Qt.AlignHCenter | Qt.AlignBottom)
                layout.addWidget(r_dial, 1, col * 2 + 1, Qt.AlignHCenter | Qt.AlignBottom)

                path = "/Mixer/Feature_LRBalance_%d" % elms[col][0]
                self.Pannings[l_dial] = [path, 1]
                self.Pannings[r_dial] = [path, 2]

                if col == 2:
                    l_dial.setDisabled(True)
                    r_dial.setDisabled(True)

            elif col == 0:
                cmb = QComboBox(parent)
                layout.addWidget(cmb, 1, col * 2, 1, 4, Qt.AlignHCenter | Qt.AlignBottom)
                for i in range(len(self.out_src[1])):
                    cmb.addItem(self.out_src[1][i], i)
                    self.Selectors[cmb] = ["/Mixer/Selector_%d" % self.out_src[0]]

            l_sld = self.getSlider(parent)
            r_sld = self.getSlider(parent)
            layout.addWidget(l_sld, 2, col * 2, Qt.AlignHCenter)
            layout.addWidget(r_sld, 2, col * 2 + 1, Qt.AlignHCenter)

            l_mute = self.getThinButton(parent, "Mute")
            r_mute = self.getThinButton(parent, "Mute")
            layout.addWidget(l_mute, 3, col * 2, Qt.AlignHCenter)
            layout.addWidget(r_mute, 3, col * 2 + 1, Qt.AlignHCenter)

            link = self.getWideButton(parent, "Link")
            layout.addWidget(link, 4, col * 2, 1, 2, Qt.AlignHCenter)

            path = "/Mixer/Feature_Volume_%d" % elms[col][0]
            self.Volumes[l_sld] = [path, 1, r_sld, l_mute, link]
            self.Volumes[r_sld] = [path, 2, l_sld, r_mute, link]

            self.Mutes[l_mute] = [r_mute, l_sld]
            self.Mutes[r_mute] = [l_mute, r_sld]

    # widget helper functions
    def getSlider(self, parent):
        sld = QSlider(parent)
        sld.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        sld.setMinimum(0)
        sld.setMaximum(99)
        sld.setPageStep(10)
        sld.setPageStep(10)
        sld.setMinimumHeight(50)
        sld.setTickInterval(25)
        sld.setTickPosition(QSlider.TicksBothSides)
        return sld

    def getDial(self, parent):
        dial = QDial(parent)
        dial.setNotchesVisible(True)
        dial.setNotchTarget(25.0)
        dial.setMaximumHeight(40)
        return dial;
 
    def getThinButton(self, parent, text):
        button = QToolButton(parent)
        button.setText(text)
        button.setCheckable(True)
        button.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        return button

    def getWideButton(self, parent, label):
        button = QToolButton(parent)
        button.setText(label)
        button.setCheckable(True)
        button.setMinimumWidth(100)
        button.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        return button

    def initValues(self):
        for ctl, params in self.Selectors.items():
           path = params[0]
           state = self.hw.getDiscrete(path)
           ctl.setCurrentIndex(state)
           ctl.activated.connect(self.updateSelector)

        for ctl, params in self.Volumes.items():
           path = params[0]
           idx  = params[1]
           pair = params[2]
           mute = params[3]
           link = params[4]

           db = self.hw.getContignuous(path, idx)
           vol = self.db2vol(db)
           ctl.setValue(int(vol))
           ctl.valueChanged.connect(self.updateVolume)

           if vol == 0:
               mute.setChecked(True)

           if idx == 2:
               pair_db = self.hw.getContignuous(path, 1)
               if pair_db == db:
                   link.setChecked(True)

        for ctl, params in self.Preamps.items():
            path = params[0]
            idx = params[1]

            vol = self.hw.getDiscrete(path, idx)
            if vol > 0:
                ctl.setChecked(True)

            ctl.clicked.connect(self.updatePreamps)

        #       Right - Center - Left
        # 0x8000 - 0x0000 - 0x0001 - 0x7FFE
        #        ..., -1, 0, +1, ...
        for ctl, params in self.Pannings.items():
            path = params[0]
            idx = params[1]

            val = self.hw.getContignuous(path, idx)
            state = -(val / 0x7FFE) * 50 + 50
            ctl.setValue(int(state))
            ctl.valueChanged.connect(self.updatePanning)

        for ctl, params in self.Mutes.items():
            ctl.clicked.connect(self.updateMute)

    # helper functions
    def vol2db(self, vol):
        return (log10(vol + 1) - 2) * 16384

    def db2vol(self, db):
        return pow(10, db / 16384 + 2) - 1

    def updateSelector(self, state):
        sender = self.sender()
        path   = self.Selectors[sender][0]
        self.hw.setDiscrete(path, state)

    def updatePreamps(self, state):
        sender = self.sender()
        path = self.Preamps[sender][0]
        idx = self.Preamps[sender][1]
        self.hw.setDiscrete(path, idx, state)

    def updateVolume(self, vol):
        sender = self.sender()
        path   = self.Volumes[sender][0]
        idx    = self.Volumes[sender][1]
        pair   = self.Volumes[sender][2]
        mute   = self.Volumes[sender][3]
        link   = self.Volumes[sender][4]

        if mute.isChecked():
                return

        db = self.vol2db(vol)
        self.hw.setContignuous(path, db, idx)

        if link.isChecked():
           pair.setValue(vol)

    def updatePanning(self, state):
        sender = self.sender()
        path = self.Pannings[sender][0]
        idx = self.Pannings[sender][1]
        val  = (state - 50) * 0x7FFE / -50
        self.hw.setContignuous(path, idx, val)

    def updateMute(self, state):
        sender = self.sender()
        pair = self.Mutes[sender][0]
        sld = self.Mutes[sender][1]

        path = self.Volumes[sld][0]
        idx = self.Volumes[sld][1]
        pair_sld = self.Volumes[sld][2]
        link = self.Volumes[sld][4]

        if state:
            db = 0x8000
            vol = 0
        else:
            db = 0x0000
            vol = 99

        self.hw.setContignuous(path, db, idx)
        sld.setValue(vol)
        sld.setDisabled(state)

        if link.isChecked():
            if idx == 1:
               idx = 2
            else:
               idx = 1
            self.hw.setContignuous(path, db, idx)
            pair.setChecked(state)
            pair_sld.setValue(vol)
            pair_sld.setDisabled(state)



#
# Copyright (C) 2005-2008 by Daniel Wagner
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget
from ffado.import_pyqt import *

from ffado.config import *

import logging
log = logging.getLogger('edirolfa101')

class EdirolFa101Control(QWidget):
    def __init__(self, parent = None):
        QWidget.__init__(self, parent)
        uicLoad("ffado/mixer/edirolfa101", self)

    def setVolumeIn1(self, vol):
        self.setValue('vol1', vol)

    def setVolumeIn2(self, vol):
        self.setValue('vol2', vol)

    def setVolumeIn3(self, vol):
        self.setValue('vol3', vol)

    def setVolumeIn4(self, vol):
        self.setValue('vol4', vol)

    def setVolumeIn5(self, vol):
        self.setValue('vol5', vol)

    def setVolumeIn6(self, vol):
        self.setValue('vol6', vol)

    def setVolumeIn7(self, vol):
        self.setValue('vol7', vol)

    def setVolumeIn8(self, vol):
        self.setValue('vol8', vol)

    def setVolumeIn9(self, vol):
        self.setValue('vol9', vol)

    def setVolumeIn10(self,vol):
        self.setValue('vol10', vol)

    def setBalanceIn1(self, bal):
        self.setValue('bal1', bal)

    def setBalanceIn2(self, bal):
        self.setValue('bal2', bal)

    def setBalanceIn3(self, bal):
        self.setValue('bal3', bal)

    def setBalanceIn4(self, bal):
        self.setValue('bal4', bal)

    def setBalanceIn5(self, bal):
        self.setValue('bal5', bal)

    def setBalanceIn6(self, bal):
        self.setValue('bal6', bal)

    def setBalanceIn7(self, bal):
        self.setValue('bal7', bal)

    def setBalanceIn8(self, bal):
        self.setValue('bal8', bal)

    def setBalanceIn9(self, bal):
        self.setValue('bal9', bal)

    def setBalanceIn10(self,bal):
        self.setValue('bal10', bal)

    def getIndexByName(self, name):
        index = int(name.lstrip('volba')) - 1
        streamingMap    = [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ]
        nonStreamingMap = [ 9, 10, 1, 2, 3, 4, 5, 6, 7, 8 ]

        if self.is_streaming:
            index = streamingMap[index]
        else:
            index = nonStreamingMap[index]
        return index
        
    def getWidgetByName(self, name):
        index = self.getIndexByName(name)
        widgetName = ''
        if name[0:3] == 'vol':
            widgetName = 'sldInput%d' % (index)
        else:
            widgetName = 'sldBal%d' % (index)
        return getattr(self, widgetName)
            
    def getFeatureByName(self, name):
        index = self.getIndexByName(name)
        featureName = ''
        if name[0:3] == 'vol':
            featureName = '/Mixer/Feature_Volume_%d' % ((index + 1) / 2)
        else:
            featureName = '/Mixer/Feature_LRBalance_%d' % ((index + 1) / 2)
        return featureName

    def getChannelIndexByName(self, name):
        index = self.getIndexByName(name)
        return ((index - 1) % 2) + 1

    def setValue(self, name, val):
        log.debug("setting %s to %d" % (name, val))
        self.updateStreamingState()
        feature = self.getFeatureByName(name)
        widget = self.getWidgetByName(name)
        channel = self.getChannelIndexByName(name)
        self.hw.setContignuous(feature, val, idx = channel)

    def updateStreamingState(self):
        ss = self.streamingstatus.selected()
        ss_txt = self.streamingstatus.getEnumLabel(ss)
        if ss_txt != 'Idle':
            self.is_streaming = True
        else:
            self.is_streaming = False
        
    def initValues(self):
        self.updateStreamingState()
        
        for i in range(1, 11):
            name = 'vol%d' % i
            feature = self.getFeatureByName(name)
            widget = self.getWidgetByName(name)
            channel = self.getChannelIndexByName(name)

            val = self.hw.getContignuous(feature, idx = channel)
            log.debug("%s value is %d" % (name , val))
            widget.setValue(val)
            
        for i in range(1, 11):
            name = 'bal%d' % i
            feature = self.getFeatureByName(name)
            widget = self.getWidgetByName(name)
            channel = self.getChannelIndexByName(name)

            val = self.hw.getContignuous(feature, idx = channel)
            # Workaround: The current value is not properly initialized
            # on the device and returns after bootup always 0.
            # Though we happen to know what the correct value should
            # be therefore we overwrite the 0 
            if channel == 1:
                val = 32512
            else:
                val = -32768
            log.debug("%s value is %d" % (name , val))
            widget.setValue(val)

# vim: et



#
# Copyright (c) 2013      by Takashi Sakamoto
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget
from ffado.import_pyqt import *

from math import log10
from ffado.config import *

import logging
log = logging.getLogger('YamahaGo')

class YamahaGo(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

    def getDisplayTitle(self):
        if self.configrom.getModelId() == 0x0010000B:
            return "GO44"
        else:
            return "GO46"

    # startup
    def initValues(self):
        uicLoad("ffado/mixer/yamahago", self)

        self.modelId = self.configrom.getModelId()
        # GO44
        if self.modelId == 0x0010000B:
            self.setWindowTitle("Yamaha GO44 Control")
            self.box_ana_out.hide()
            self.is46 = False
        # GO46
        elif self.modelId == 0x0010000C:
            self.setWindowTitle("Yamaha GO46 Control")
            self.box_ana_in_12_level.hide()
            self.is46 = True

        # common
        self.VolumeControls = {
            self.sld_mixer_strm_in_1:
                ('/Mixer/Feature_Volume_3', 1,
                 self.sld_mixer_strm_in_2, 2, self.link_mixer_strm_in_12),
            self.sld_mixer_strm_in_2:
                ('/Mixer/Feature_Volume_3', 2,
                 self.sld_mixer_strm_in_1, 1, self.link_mixer_strm_in_12),
            self.sld_mixer_strm_in_3:
                ('/Mixer/Feature_Volume_4', 1,
                 self.sld_mixer_strm_in_4, 2, self.link_mixer_strm_in_34),
            self.sld_mixer_strm_in_4:
                ('/Mixer/Feature_Volume_4', 2,
                 self.sld_mixer_strm_in_3, 1, self.link_mixer_strm_in_34),
            self.sld_mixer_strm_in_5:
                ('/Mixer/Feature_Volume_5', 1,
                 self.sld_mixer_strm_in_6, 2, self.link_mixer_strm_in_56),
            self.sld_mixer_strm_in_6:
                ('/Mixer/Feature_Volume_5', 2,
                 self.sld_mixer_strm_in_5, 1, self.link_mixer_strm_in_56),
            self.sld_mixer_ana_in_1:
                ('/Mixer/Feature_Volume_6', 1,
                 self.sld_mixer_ana_in_2, 2, self.link_mixer_ana_in_12),
            self.sld_mixer_ana_in_2:
                ('/Mixer/Feature_Volume_6', 2,
                 self.sld_mixer_ana_in_1, 1, self.link_mixer_ana_in_12),
            self.sld_mixer_dig_in_1:
                ('/Mixer/Feature_Volume_7', 1,
                 self.sld_mixer_dig_in_2, 2, self.link_mixer_dig_in_12),
            self.sld_mixer_dig_in_2:
                ('/Mixer/Feature_Volume_7', 2,
                 self.sld_mixer_dig_in_1, 1, self.link_mixer_dig_in_12)
        }
        self.JackSourceSelectors = {
            self.cmb_ana_out_12_route:    '/Mixer/Selector_1',
            self.cmb_ana_out_34_route:    '/Mixer/Selector_2',
            self.cmb_dig_out_12_route:    '/Mixer/Selector_3'
            }

        if not self.is46:
            # volume for mixer output
            self.VolumeControls[self.sld_mixer_out_1] = (
                '/Mixer/Feature_Volume_1', 1,
                self.sld_mixer_out_2, 2, self.link_mixer_out_12
            )
            self.VolumeControls[self.sld_mixer_out_2] = (
                '/Mixer/Feature_Volume_1', 2,
                self.sld_mixer_out_1, 1, self.link_mixer_out_12
            )
            # analog out 3/4 is headphone out 1/2
            self.label_ana_out_34_route.setText("Headphone out 1/2")
        else:
            # volume for mixer output
            self.VolumeControls[self.sld_mixer_out_1] = (
                '/Mixer/Feature_Volume_2', 1,
                self.sld_mixer_out_2, 2, self.link_mixer_out_12
            )
            self.VolumeControls[self.sld_mixer_out_2] = (
                '/Mixer/Feature_Volume_2', 2,
                self.sld_mixer_out_1, 1, self.link_mixer_out_12
            )
            # volume for analog output
            self.VolumeControls[self.sld_ana_out_1] = (
                '/Mixer/Feature_Volume_1', 1,
                self.sld_ana_out_2, 2, self.link_ana_out_12
            )
            self.VolumeControls[self.sld_ana_out_2] = (
                '/Mixer/Feature_Volume_1', 2,
                self.sld_ana_out_1, 1, self.link_ana_out_12
            )
            self.VolumeControls[self.sld_ana_out_3] = (
                '/Mixer/Feature_Volume_1', 3,
                self.sld_ana_out_4, 4, self.link_ana_out_34
            )
            self.VolumeControls[self.sld_ana_out_4] = (
                '/Mixer/Feature_Volume_1', 4,
                self.sld_ana_out_3, 3, self.link_ana_out_34
            )

        # gain control
        for ctl, params in list(self.VolumeControls.items()):
            path = params[0]
            idx = params[1]

            db = self.hw.getContignuous(path, idx)
            vol = self.db2vol(db)
            ctl.setValue(int(vol))

            link = params[4]

            pvol = self.db2vol(db)

            if pvol == vol:
                link.setChecked(True)

            ctl.valueChanged.connect(self.updateVolume)

        # source selector for jack output
        for ctl, param in list(self.JackSourceSelectors.items()):
            state = self.hw.getDiscrete(param)
            ctl.setCurrentIndex(state)

            ctl.activated.connect(self.updateSelector)

        if not self.is46:
            self.cmb_ana_in_12_level.activated.connect(self.updateMicLevel)

    # helper functions
    def vol2db(self, vol):
        return (log10(vol + 1) - 2) * 16384
    def db2vol(self, db):
        return pow(10, db / 16384 + 2) - 1

    def updateMicLevel(self, state):
        if state == 0:
            level = 0
        elif state == 1:
            level = -1535
        else:
            level = -3583
        self.hw.setContignuous('/Mixer/Feature_Volume_2', level)

    def updateVolume(self, vol):
        sender = self.sender()
        path = self.VolumeControls[sender][0]
        idx = self.VolumeControls[sender][1]
        pair = self.VolumeControls[sender][2]
        link = self.VolumeControls[sender][4]

        db = self.vol2db(vol)
        self.hw.setContignuous(path, db, idx)

        if link.isChecked():
            pair.setValue(vol)

    def updateSelector(self, state):
        sender = self.sender()
        path = self.JackSourceSelectors[sender]
        self.hw.setDiscrete(path, state)



#
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget, QHBoxLayout
from ffado.import_pyqt import *

from ffado.config import *
from ffado.mixer.saffire_base import SaffireMixerBase

import logging
log = logging.getLogger('saffire')

#MIXER LAYOUT:
#
#   |-- Out9/10--| |-- Out1/2 --| |-- Out3/4 --| |-- Out5/6 --| |-- Out7/8 --|
#P5  0:    0/    0  1:  110/  110  2:    0/    0  3:    0/    0  4:    0/    0
#P1  5:    0/    0  6:32767/32767  7:    0/    0  8:    0/    0  9:    0/    0
#P2 10:    0/    0 11:    0/    0 12:32767/32767 13:    0/    0 14:    0/    0
#P3 15:    0/    0 16:    0/    0 17:    0/    0 18:32767/32767 19:    0/    0
#P4 20:    0/    0 21:    0/    0 22:    0/    0 23:    0/    0 24:32767/32767
#R1 25:    0/    0 26:    0/    0 27:    0/    0 28:    0/    0 29:    0/    0
#R2 30:    0/    0 31:    0/    0 32:    0/    0 33:    0/    0 34:    0/    0
#Fx 35:    0/    0 36:    0/    0 37:    0/    0 38:    0/    0 39:    0/    0
#
#P5: DAW ch 9/10
#P1: DAW ch 1/2
#P2: DAW ch 3/4
#P3: DAW ch 5/6
#P4: DAW ch 7/8
#R1: HW INPUT ch 1/2
#R2: HW INPUT ch 3/4
#Fx: reverb/fx return

class Saffire(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self, parent)

        self.mono_mode = False
        self.is_saffire_le = False

        # make a layout
        self.layout = QHBoxLayout()
        self.setLayout(self.layout)

    def show(self):
        self.selectCorrectMode()
        QWidget.show(self)

    def getMonoMode(self):
        return self.hw.getDiscrete('/Mixer/MonoMode')
    def setMonoMode(self, mode):
        if mode:
            self.hw.setDiscrete('/Mixer/MonoMode', 1)
        else:
            self.hw.setDiscrete('/Mixer/MonoMode', 0)
        self.mono_mode = self.getMonoMode()

    def getDisplayTitle(self):
        if self.is_saffire_le:
            return "SaffireLE"
        else:
            return "Saffire"

    def selectCorrectMode(self):
        if self.is_saffire_le:
            if self.samplerate <= 48000:
                log.debug("large")
                self.small.hide()
                self.large.initValues()
                self.large.show()
            else:
                log.debug("small")
                self.large.hide()
                self.small.initValues()
                self.small.show()
        else:
            if self.mono_mode:
                self.stereo.hide()
                self.mono.initValues()
                self.mono.show()
            else:
                self.mono.hide()
                self.stereo.initValues()
                self.stereo.show()

    def initValues(self):
        selected = self.samplerateselect.selected()
        label = self.samplerateselect.getEnumLabel( selected )
        try:
            self.samplerate = int(label)
        except:
            # FIXME: this should be handled properly
            self.samplerate = 44100

        # Saffire:        0x130e010001????
        # SaffireLE:    0x130e010004????
        if int(self.configrom.getGUID(), 16) >= 0x130e0100040000:
            self.is_saffire_le = True
            log.debug("Found SaffireLE GUID")
        else:
            self.is_saffire_le = False
            log.debug("Found Saffire GUID")

        # init depending on what device we have
        # and what mode it is in
        if self.is_saffire_le:
            # create the child widgets
            self.small = SaffireLEMixerSmall(self)
            self.layout.addWidget(self.small)
            self.large = SaffireLEMixerLarge(self)
            self.layout.addWidget(self.large)

            self.small.hw = self.hw
            self.small.configrom = self.configrom

            self.large.hw = self.hw
            self.large.configrom = self.configrom
        else:
            # create the child widgets
            self.mono = SaffireMixerMono(self)
            self.layout.addWidget(self.mono)
            self.stereo = SaffireMixerStereo(self)
            self.layout.addWidget(self.stereo)

            self.mono_mode = self.getMonoMode()

            self.mono.hw = self.hw
            self.mono.configrom = self.configrom

            self.stereo.hw = self.hw
            self.stereo.configrom = self.configrom

        self.selectCorrectMode()

    def polledUpdate(self):
        if self.is_saffire_le:
            if self.samplerate <= 48000:
                self.large.polledUpdate()
            else:
                self.small.polledUpdate()
        else:
            if self.mono_mode:
                self.mono.polledUpdate()
            else:
                self.stereo.polledUpdate()

class SaffireMixerStereo(QWidget, SaffireMixerBase):
    def __init__(self,parent = None):
        self.my_parent = parent
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/saffire_stereo", self)
        SaffireMixerBase.__init__(self)
        self.btnRefresh.clicked.connect(self.updateValues)
        self.btnSwitchStereoMode.clicked.connect(self.switchStereoMode)

        self.VolumeControls={
                self.sldPC910Out910: ['/Mixer/MatrixMixerStereo', 0, 0],
                self.sldPC910Out12: ['/Mixer/MatrixMixerStereo', 0, 1],
                self.sldPC910Out34: ['/Mixer/MatrixMixerStereo', 0, 2],
                self.sldPC910Out56: ['/Mixer/MatrixMixerStereo', 0, 3],
                self.sldPC910Out78: ['/Mixer/MatrixMixerStereo', 0, 4],
                self.sldPC12Out910: ['/Mixer/MatrixMixerStereo', 1, 0],
                self.sldPC12Out12: ['/Mixer/MatrixMixerStereo', 1, 1],
                self.sldPC12Out34: ['/Mixer/MatrixMixerStereo', 1, 2],
                self.sldPC12Out56: ['/Mixer/MatrixMixerStereo', 1, 3],
                self.sldPC12Out78: ['/Mixer/MatrixMixerStereo', 1, 4],
                self.sldPC34Out910: ['/Mixer/MatrixMixerStereo', 2, 0],
                self.sldPC34Out12: ['/Mixer/MatrixMixerStereo', 2, 1],
                self.sldPC34Out34: ['/Mixer/MatrixMixerStereo', 2, 2],
                self.sldPC34Out56: ['/Mixer/MatrixMixerStereo', 2, 3],
                self.sldPC34Out78: ['/Mixer/MatrixMixerStereo', 2, 4],
                self.sldPC56Out910: ['/Mixer/MatrixMixerStereo', 3, 0],
                self.sldPC56Out12: ['/Mixer/MatrixMixerStereo', 3, 1],
                self.sldPC56Out34: ['/Mixer/MatrixMixerStereo', 3, 2],
                self.sldPC56Out56: ['/Mixer/MatrixMixerStereo', 3, 3],
                self.sldPC56Out78: ['/Mixer/MatrixMixerStereo', 3, 4],
                self.sldPC78Out910: ['/Mixer/MatrixMixerStereo', 4, 0],
                self.sldPC78Out12: ['/Mixer/MatrixMixerStereo', 4, 1],
                self.sldPC78Out34: ['/Mixer/MatrixMixerStereo', 4, 2],
                self.sldPC78Out56: ['/Mixer/MatrixMixerStereo', 4, 3],
                self.sldPC78Out78: ['/Mixer/MatrixMixerStereo', 4, 4],

                self.sldIN12Out910: ['/Mixer/MatrixMixerStereo', 5, 0],
                self.sldIN12Out12: ['/Mixer/MatrixMixerStereo', 5, 1],
                self.sldIN12Out34: ['/Mixer/MatrixMixerStereo', 5, 2],
                self.sldIN12Out56: ['/Mixer/MatrixMixerStereo', 5, 3],
                self.sldIN12Out78: ['/Mixer/MatrixMixerStereo', 5, 4],
                self.sldIN34Out910: ['/Mixer/MatrixMixerStereo', 6, 0],
                self.sldIN34Out12: ['/Mixer/MatrixMixerStereo', 6, 1],
                self.sldIN34Out34: ['/Mixer/MatrixMixerStereo', 6, 2],
                self.sldIN34Out56: ['/Mixer/MatrixMixerStereo', 6, 3],
                self.sldIN34Out78: ['/Mixer/MatrixMixerStereo', 6, 4],

                self.sldFXOut910: ['/Mixer/MatrixMixerStereo', 7, 0],
                self.sldFXOut12: ['/Mixer/MatrixMixerStereo', 7, 1],
                self.sldFXOut34: ['/Mixer/MatrixMixerStereo', 7, 2],
                self.sldFXOut56: ['/Mixer/MatrixMixerStereo', 7, 3],
                self.sldFXOut78: ['/Mixer/MatrixMixerStereo', 7, 4],
                }


        # First column is the DBUS subpath of the control.
        # Second column is a list of linked controls that should
        # be rewritten whenever this control is updated
        self.SelectorControls={
                self.chkSpdifSwitch:    ['/Mixer/SpdifSwitch'],
                self.chkOut12Mute:      ['/Mixer/Out12Mute', [self.chkOut12HwCtrl]],
                self.chkOut12HwCtrl:    ['/Mixer/Out12HwCtrl'],
                self.chkOut12Dim:       ['/Mixer/Out12Dim'],
                self.chkOut34Mute:      ['/Mixer/Out34Mute', [self.chkOut34HwCtrl]],
                self.chkOut34HwCtrl:    ['/Mixer/Out34HwCtrl'],
                self.chkOut56Mute:      ['/Mixer/Out56Mute', [self.chkOut56HwCtrl]],
                self.chkOut56HwCtrl:    ['/Mixer/Out56HwCtrl'],
                self.chkOut78Mute:      ['/Mixer/Out78Mute', [self.chkOut78HwCtrl]],
                self.chkOut78HwCtrl:    ['/Mixer/Out78HwCtrl'],
                self.chkOut910Mute:     ['/Mixer/Out910Mute'],
                }

        self.VolumeControlsLowRes={
                self.sldOut12Level:      ['/Mixer/Out12Level'],
                self.sldOut34Level:      ['/Mixer/Out34Level'],
                self.sldOut56Level:      ['/Mixer/Out56Level'],
                self.sldOut78Level:      ['/Mixer/Out78Level'],
                }

        self.TriggerButtonControls={
                self.btnSaveSettings:      ['/Mixer/SaveSettings'],
        }

        self.TextControls={
        }

        self.saveTextControls={
        }

        self.ComboControls={
        }

    def polledUpdate(self):
        self.polledUpdateHwCtrl(self.chkOut12HwCtrl, self.sldOut12Level)
        self.polledUpdateHwCtrl(self.chkOut34HwCtrl, self.sldOut34Level)
        self.polledUpdateHwCtrl(self.chkOut56HwCtrl, self.sldOut56Level)
        self.polledUpdateHwCtrl(self.chkOut78HwCtrl, self.sldOut78Level)

    def polledUpdateHwCtrl(self, selector, volctrl):
        state = selector.isChecked()
        if state:
            self.polledUpdateVolumeLowRes('/Mixer/MonitorDial', volctrl, 64)
            volctrl.setEnabled(False)
        else:
            volctrl.setEnabled(True)

    def updateMatrixVolume(self,a0):
        SaffireMixerBase.updateMatrixVolume(self,a0)
    def updateLowResVolume(self,a0):
        SaffireMixerBase.updateLowResVolume(self,a0)
    def updateSelector(self,a0):
        SaffireMixerBase.updateSelector(self,a0)
    def triggerButton(self):
        SaffireMixerBase.triggerButton(self)
    def saveText(self):
        SaffireMixerBase.saveText(self)
    def initCombo(self, combo):
        SaffireMixerBase.initCombo(self,combo)
    def selectCombo(self, mode):
        SaffireMixerBase.selectCombo(self,mode)

    def updateValues(self):
        SaffireMixerBase.updateValues(self)
    def switchStereoMode(self):
        log.debug("should switch to mono mode")
        self.my_parent.setMonoMode(1)
        self.my_parent.selectCorrectMode()

class SaffireMixerMono(QWidget, SaffireMixerBase):
    def __init__(self,parent = None):
        self.my_parent = parent
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/saffire_mono", self)
        SaffireMixerBase.__init__(self)
        self.btnRefresh.clicked.connect(self.updateValues)
        self.btnSwitchStereoMode.clicked.connect(self.switchStereoMode)

        self.VolumeControls={
                self.sldIN1Out910: ['/Mixer/MatrixMixerMono', 0, 0],
                self.sldIN1Out12: ['/Mixer/MatrixMixerMono', 0, 1],
                self.sldIN1Out34: ['/Mixer/MatrixMixerMono', 0, 2],
                self.sldIN1Out56: ['/Mixer/MatrixMixerMono', 0, 3],
                self.sldIN1Out78: ['/Mixer/MatrixMixerMono', 0, 4],
                self.sldIN3Out910: ['/Mixer/MatrixMixerMono', 1, 0],
                self.sldIN3Out12: ['/Mixer/MatrixMixerMono', 1, 1],
                self.sldIN3Out34: ['/Mixer/MatrixMixerMono', 1, 2],
                self.sldIN3Out56: ['/Mixer/MatrixMixerMono', 1, 3],
                self.sldIN3Out78: ['/Mixer/MatrixMixerMono', 1, 4],
                self.sldFX1Out910: ['/Mixer/MatrixMixerMono', 2, 0],
                self.sldFX1Out12: ['/Mixer/MatrixMixerMono', 2, 1],
                self.sldFX1Out34: ['/Mixer/MatrixMixerMono', 2, 2],
                self.sldFX1Out56: ['/Mixer/MatrixMixerMono', 2, 3],
                self.sldFX1Out78: ['/Mixer/MatrixMixerMono', 2, 4],
                self.sldIN2Out910: ['/Mixer/MatrixMixerMono', 3, 0],
                self.sldIN2Out12: ['/Mixer/MatrixMixerMono', 3, 1],
                self.sldIN2Out34: ['/Mixer/MatrixMixerMono', 3, 2],
                self.sldIN2Out56: ['/Mixer/MatrixMixerMono', 3, 3],
                self.sldIN2Out78: ['/Mixer/MatrixMixerMono', 3, 4],
                self.sldIN4Out910: ['/Mixer/MatrixMixerMono', 4, 0],
                self.sldIN4Out12: ['/Mixer/MatrixMixerMono', 4, 1],
                self.sldIN4Out34: ['/Mixer/MatrixMixerMono', 4, 2],
                self.sldIN4Out56: ['/Mixer/MatrixMixerMono', 4, 3],
                self.sldIN4Out78: ['/Mixer/MatrixMixerMono', 4, 4],
                self.sldFX2Out910: ['/Mixer/MatrixMixerMono', 5, 0],
                self.sldFX2Out12: ['/Mixer/MatrixMixerMono', 5, 1],
                self.sldFX2Out34: ['/Mixer/MatrixMixerMono', 5, 2],
                self.sldFX2Out56: ['/Mixer/MatrixMixerMono', 5, 3],
                self.sldFX2Out78: ['/Mixer/MatrixMixerMono', 5, 4],

                self.sldPC910Out910: ['/Mixer/MatrixMixerMono', 6, 0],
                self.sldPC910Out12: ['/Mixer/MatrixMixerMono', 6, 1],
                self.sldPC910Out34: ['/Mixer/MatrixMixerMono', 6, 2],
                self.sldPC910Out56: ['/Mixer/MatrixMixerMono', 6, 3],
                self.sldPC910Out78: ['/Mixer/MatrixMixerMono', 6, 4],
                self.sldPC12Out910: ['/Mixer/MatrixMixerMono', 7, 0],
                self.sldPC12Out12: ['/Mixer/MatrixMixerMono', 7, 1],
                self.sldPC12Out34: ['/Mixer/MatrixMixerMono', 7, 2],
                self.sldPC12Out56: ['/Mixer/MatrixMixerMono', 7, 3],
                self.sldPC12Out78: ['/Mixer/MatrixMixerMono', 7, 4],
                self.sldPC34Out910: ['/Mixer/MatrixMixerMono', 8, 0],
                self.sldPC34Out12: ['/Mixer/MatrixMixerMono', 8, 1],
                self.sldPC34Out34: ['/Mixer/MatrixMixerMono', 8, 2],
                self.sldPC34Out56: ['/Mixer/MatrixMixerMono', 8, 3],
                self.sldPC34Out78: ['/Mixer/MatrixMixerMono', 8, 4],
                self.sldPC56Out910: ['/Mixer/MatrixMixerMono', 9, 0],
                self.sldPC56Out12: ['/Mixer/MatrixMixerMono', 9, 1],
                self.sldPC56Out34: ['/Mixer/MatrixMixerMono', 9, 2],
                self.sldPC56Out56: ['/Mixer/MatrixMixerMono', 9, 3],
                self.sldPC56Out78: ['/Mixer/MatrixMixerMono', 9, 4],
                self.sldPC78Out910: ['/Mixer/MatrixMixerMono', 10, 0],
                self.sldPC78Out12: ['/Mixer/MatrixMixerMono', 10, 1],
                self.sldPC78Out34: ['/Mixer/MatrixMixerMono', 10, 2],
                self.sldPC78Out56: ['/Mixer/MatrixMixerMono', 10, 3],
                self.sldPC78Out78: ['/Mixer/MatrixMixerMono', 10, 4],


                }


        # First column is the DBUS subpath of the control.
        # Second column is a list of linked controls that should
        # be rewritten whenever this control is updated
        self.SelectorControls={
                self.chkSpdifSwitch:    ['/Mixer/SpdifSwitch'],
                self.chkOut12Mute:      ['/Mixer/Out12Mute', [self.chkOut12HwCtrl]],
                self.chkOut12HwCtrl:    ['/Mixer/Out12HwCtrl'],
                self.chkOut12Dim:       ['/Mixer/Out12Dim'],
                self.chkOut34Mute:      ['/Mixer/Out34Mute', [self.chkOut34HwCtrl]],
                self.chkOut34HwCtrl:    ['/Mixer/Out34HwCtrl'],
                self.chkOut56Mute:      ['/Mixer/Out56Mute', [self.chkOut56HwCtrl]],
                self.chkOut56HwCtrl:    ['/Mixer/Out56HwCtrl'],
                self.chkOut78Mute:      ['/Mixer/Out78Mute', [self.chkOut78HwCtrl]],
                self.chkOut78HwCtrl:    ['/Mixer/Out78HwCtrl'],
                self.chkOut910Mute:     ['/Mixer/Out910Mute'],
                }

        self.VolumeControlsLowRes={
                self.sldOut12Level:      ['/Mixer/Out12Level'],
                self.sldOut34Level:      ['/Mixer/Out34Level'],
                self.sldOut56Level:      ['/Mixer/Out56Level'],
                self.sldOut78Level:      ['/Mixer/Out78Level'],
                }

        self.TriggerButtonControls={
                self.btnSaveSettings:      ['/Mixer/SaveSettings'],
        }

        self.TextControls={
        }

        self.saveTextControls={
        }

        self.ComboControls={
        }

    def polledUpdate(self):
        self.polledUpdateHwCtrl(self.chkOut12HwCtrl, self.sldOut12Level)
        self.polledUpdateHwCtrl(self.chkOut34HwCtrl, self.sldOut34Level)
        self.polledUpdateHwCtrl(self.chkOut56HwCtrl, self.sldOut56Level)
        self.polledUpdateHwCtrl(self.chkOut78HwCtrl, self.sldOut78Level)

    def polledUpdateHwCtrl(self, selector, volctrl):
        state = selector.isChecked()
        if state:
            self.polledUpdateVolumeLowRes('/Mixer/MonitorDial', volctrl, 4)
            volctrl.setEnabled(False)
        else:
            volctrl.setEnabled(True)

    def updateMatrixVolume(self,a0):
        SaffireMixerBase.updateMatrixVolume(self,a0)
    def updateLowResVolume(self,a0):
        SaffireMixerBase.updateLowResVolume(self,a0)
    def updateSelector(self,a0):
        SaffireMixerBase.updateSelector(self,a0)
    def triggerButton(self):
        SaffireMixerBase.triggerButton(self)
    def saveText(self):
        SaffireMixerBase.saveText(self)
    def initCombo(self, combo):
        SaffireMixerBase.initCombo(self,combo)
    def selectCombo(self, mode):
        SaffireMixerBase.selectCombo(self,mode)

    def updateValues(self):
        SaffireMixerBase.updateValues(self)

    def switchStereoMode(self):
        log.debug("should switch to stereo mode")
        self.my_parent.setMonoMode(0)
        self.my_parent.selectCorrectMode()

class SaffireLEMixerLarge(QWidget, SaffireMixerBase):
    def __init__(self,parent = None):
        self.my_parent = parent
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/saffirele_large", self)
        SaffireMixerBase.__init__(self)

        log.debug("Init large Saffire LE mixer window")

        self.VolumeControls={
                self.sldIN1Out1: ['/Mixer/LEMix48', 0, 0],
                self.sldIN1Out2: ['/Mixer/LEMix48', 0, 1],
                self.sldIN1Out3: ['/Mixer/LEMix48', 0, 2],
                self.sldIN1Out4: ['/Mixer/LEMix48', 0, 3],
                self.sldIN2Out1: ['/Mixer/LEMix48', 1, 0],
                self.sldIN2Out2: ['/Mixer/LEMix48', 1, 1],
                self.sldIN2Out3: ['/Mixer/LEMix48', 1, 2],
                self.sldIN2Out4: ['/Mixer/LEMix48', 1, 3],
                self.sldIN3Out1: ['/Mixer/LEMix48', 2, 0],
                self.sldIN3Out2: ['/Mixer/LEMix48', 2, 1],
                self.sldIN3Out3: ['/Mixer/LEMix48', 2, 2],
                self.sldIN3Out4: ['/Mixer/LEMix48', 2, 3],
                self.sldIN4Out1: ['/Mixer/LEMix48', 3, 0],
                self.sldIN4Out2: ['/Mixer/LEMix48', 3, 1],
                self.sldIN4Out3: ['/Mixer/LEMix48', 3, 2],
                self.sldIN4Out4: ['/Mixer/LEMix48', 3, 3],
                self.sldSPDIF1Out1: ['/Mixer/LEMix48', 4, 0],
                self.sldSPDIF1Out2: ['/Mixer/LEMix48', 4, 1],
                self.sldSPDIF1Out3: ['/Mixer/LEMix48', 4, 2],
                self.sldSPDIF1Out4: ['/Mixer/LEMix48', 4, 3],
                self.sldSPDIF2Out1: ['/Mixer/LEMix48', 5, 0],
                self.sldSPDIF2Out2: ['/Mixer/LEMix48', 5, 1],
                self.sldSPDIF2Out3: ['/Mixer/LEMix48', 5, 2],
                self.sldSPDIF2Out4: ['/Mixer/LEMix48', 5, 3],
                
                self.sldPC1Out1: ['/Mixer/LEMix48', 6, 0],
                self.sldPC1Out2: ['/Mixer/LEMix48', 6, 1],
                self.sldPC1Out3: ['/Mixer/LEMix48', 6, 2],
                self.sldPC1Out4: ['/Mixer/LEMix48', 6, 3],
                self.sldPC2Out1: ['/Mixer/LEMix48', 7, 0],
                self.sldPC2Out2: ['/Mixer/LEMix48', 7, 1],
                self.sldPC2Out3: ['/Mixer/LEMix48', 7, 2],
                self.sldPC2Out4: ['/Mixer/LEMix48', 7, 3],
                self.sldPC3Out1: ['/Mixer/LEMix48', 8, 0],
                self.sldPC3Out2: ['/Mixer/LEMix48', 8, 1],
                self.sldPC3Out3: ['/Mixer/LEMix48', 8, 2],
                self.sldPC3Out4: ['/Mixer/LEMix48', 8, 3],
                self.sldPC4Out1: ['/Mixer/LEMix48', 9, 0],
                self.sldPC4Out2: ['/Mixer/LEMix48', 9, 1],
                self.sldPC4Out3: ['/Mixer/LEMix48', 9, 2],
                self.sldPC4Out4: ['/Mixer/LEMix48', 9, 3],
                self.sldPC5Out1: ['/Mixer/LEMix48', 10, 0],
                self.sldPC5Out2: ['/Mixer/LEMix48', 10, 1],
                self.sldPC5Out3: ['/Mixer/LEMix48', 10, 2],
                self.sldPC5Out4: ['/Mixer/LEMix48', 10, 3],
                self.sldPC6Out1: ['/Mixer/LEMix48', 11, 0],
                self.sldPC6Out2: ['/Mixer/LEMix48', 11, 1],
                self.sldPC6Out3: ['/Mixer/LEMix48', 11, 2],
                self.sldPC6Out4: ['/Mixer/LEMix48', 11, 3],
                self.sldPC7Out1: ['/Mixer/LEMix48', 12, 0],
                self.sldPC7Out2: ['/Mixer/LEMix48', 12, 1],
                self.sldPC7Out3: ['/Mixer/LEMix48', 12, 2],
                self.sldPC7Out4: ['/Mixer/LEMix48', 12, 3],
                self.sldPC8Out1: ['/Mixer/LEMix48', 13, 0],
                self.sldPC8Out2: ['/Mixer/LEMix48', 13, 1],
                self.sldPC8Out3: ['/Mixer/LEMix48', 13, 2],
                self.sldPC8Out4: ['/Mixer/LEMix48', 13, 3],
                }

        self.SelectorControls={
                self.chkOut12Mute:          ['/Mixer/Out12Mute'],
                self.chkOut12HwCtrl:        ['/Mixer/Out12HwCtrl'],
                self.chkOut34Mute:          ['/Mixer/Out34Mute'],
                self.chkOut34HwCtrl:        ['/Mixer/Out34HwCtrl'],
                self.chkOut56Mute:          ['/Mixer/Out56Mute'],
                self.chkOut56HwCtrl:        ['/Mixer/Out56HwCtrl'],
                self.chkSPDIFTransparent:   ['/Mixer/SpdifTransparent'],
                self.chkMIDITru:            ['/Mixer/MidiThru'],
                self.chkHighGain3:          ['/Mixer/HighGainLine3'],
                self.chkHighGain4:          ['/Mixer/HighGainLine4'],
                }

        self.VolumeControlsLowRes={
                self.sldOut12Level:      ['/Mixer/Out12Level'],
                self.sldOut34Level:      ['/Mixer/Out34Level'],
                self.sldOut56Level:      ['/Mixer/Out56Level'],
                }

        self.TriggerButtonControls={
            self.btnSaveSettings:        ['/Mixer/SaveSettings'],
        }

        self.TextControls={
        }

        self.saveTextControls={
        }

        self.ComboControls={
        }

    def polledUpdate(self):
        #fixme do what it takes to make the gui follow the front panel dial
        pass

    def updateMatrixVolume(self,a0):
        SaffireMixerBase.updateMatrixVolume(self,a0)
    def updateLowResVolume(self,a0):
        SaffireMixerBase.updateLowResVolume(self,a0)
    def updateSelector(self,a0):
        SaffireMixerBase.updateSelector(self,a0)
    def triggerButton(self):
        SaffireMixerBase.triggerButton(self)
    def saveText(self):
        SaffireMixerBase.saveText(self)
    def initCombo(self, combo):
        SaffireMixerBase.initCombo(self,combo)
    def selectCombo(self, mode):
        SaffireMixerBase.selectCombo(self,mode)

    def updateValues(self):
        SaffireMixerBase.updateValues(self)

class SaffireLEMixerSmall(QWidget, SaffireMixerBase):
    def __init__(self,parent = None):
        self.my_parent = parent
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/saffirele_small", self)
        SaffireMixerBase.__init__(self)

        log.debug("Init small Saffire LE mixer window")

        self.VolumeControls={
                self.sldIN1RecMix:    ['/Mixer/LEMix96', 0, 4],
                self.sldIN2RecMix:    ['/Mixer/LEMix96', 1, 4],
                self.sldIN3RecMix:    ['/Mixer/LEMix96', 2, 4],
                self.sldIN4RecMix:    ['/Mixer/LEMix96', 3, 4],
                self.sldSPDIF1RecMix: ['/Mixer/LEMix96', 4, 4],
                self.sldSPDIF2RecMix: ['/Mixer/LEMix96', 5, 4],

                self.sldPC1Out1: ['/Mixer/LEMix96', 6, 0],
                self.sldPC1Out2: ['/Mixer/LEMix96', 6, 1],
                self.sldPC1Out3: ['/Mixer/LEMix96', 6, 2],
                self.sldPC1Out4: ['/Mixer/LEMix96', 6, 3],
                self.sldPC2Out1: ['/Mixer/LEMix96', 7, 0],
                self.sldPC2Out2: ['/Mixer/LEMix96', 7, 1],
                self.sldPC2Out3: ['/Mixer/LEMix96', 7, 2],
                self.sldPC2Out4: ['/Mixer/LEMix96', 7, 3],
                }

        self.SelectorControls={
                self.chkOut12Mute:          ['/Mixer/Out12Mute'],
                self.chkOut12HwCtrl:        ['/Mixer/Out12HwCtrl'],
                self.chkOut34Mute:          ['/Mixer/Out34Mute'],
                self.chkOut34HwCtrl:        ['/Mixer/Out34HwCtrl'],
                self.chkOut56Mute:          ['/Mixer/Out56Mute'],
                self.chkOut56HwCtrl:        ['/Mixer/Out56HwCtrl'],
                self.chkSPDIFTransparent:   ['/Mixer/SpdifTransparent'],
                self.chkMIDITru:            ['/Mixer/MidiThru'],
                self.chkHighGain3:          ['/Mixer/HighGainLine3'],
                self.chkHighGain4:          ['/Mixer/HighGainLine4'],
                }

        self.VolumeControlsLowRes={
                self.sldOut12Level:      ['/Mixer/Out12Level'],
                self.sldOut34Level:      ['/Mixer/Out34Level'],
                self.sldOut56Level:      ['/Mixer/Out56Level'],
                }

        self.TriggerButtonControls={
            self.btnSaveSettings:        ['/Mixer/SaveSettings'],
        }

        self.TextControls={
        }

        self.saveTextControls={
        }

        self.ComboControls={
        }

    def polledUpdate(self):
        #fixme do what it takes to make the gui follow the front panel dial
        pass

    def updateMatrixVolume(self,a0):
        SaffireMixerBase.updateMatrixVolume(self,a0)
    def updateLowResVolume(self,a0):
        SaffireMixerBase.updateLowResVolume(self,a0)
    def updateSelector(self,a0):
        SaffireMixerBase.updateSelector(self,a0)
    def triggerButton(self):
        SaffireMixerBase.triggerButton(self)
    def saveText(self):
        SaffireMixerBase.saveText(self)
    def initCombo(self, combo):
        SaffireMixerBase.initCombo(self,combo)
    def selectCombo(self, mode):
        SaffireMixerBase.selectCombo(self,mode)

    def updateValues(self):
        SaffireMixerBase.updateValues(self)

# vim: et



#
# Copyright (C) 2009-2010 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4 import QtGui, QtCore, Qt
# from PyQt4.QtGui import QWidget, QScrollArea
from ffado.import_pyqt import *

import dbus

import ffado.config
from ffado.widgets.matrixmixer import MatrixMixer
from ffado.widgets.crossbarrouter import *
from ffado.mixer.generic_dice_eap import *

from ffado.config import *

class BooleanControl:
    def __init__(self, hw, path):
        if ffado.config.bypassdbus:
            self.value = False
            return
        self.iface = dbus.Interface(
                hw.bus.get_object(hw.servername, path),
                dbus_interface="org.ffado.Control.Element.Boolean")
        self.value = self.iface.selected()

    def selected(self):
        return self.value

    def select(self, n):
        if self.iface.select(n):
            self.value = n
            return True
        return False

class DiscreteControl:
    def __init__(self, hw, path):
        if ffado.config.bypassdbus:
            self.value = 0
            return
        self.iface = dbus.Interface(
                hw.bus.get_object(hw.servername, path),
                dbus_interface="org.ffado.Control.Element.Discrete")
        self.value = self.iface.getValue()

    def getvalue(self):
        return self.value

    def setvalue(self, v):
        if v != self.value:
            self.iface.setValue(v)
            self.value = v

class Profire2626(Generic_Dice_EAP):
    def __init__(self, parent=None):
        Generic_Dice_EAP.__init__(self, parent)

    def buildMixer(self):
        #print( self.hw )
        #print( self.hw.getText("/Generic/Nickname") )
        Generic_Dice_EAP.buildMixer(self)

        widget = QWidget()
        uicLoad("ffado/mixer/profire2626_settings.ui", widget)

        # Add Settings to ffado-mixer panels
        scrollarea = QScrollArea(self.tabs)
        scrollarea.setWidgetResizable(False)
        scrollarea.setWidget(widget)
        self.tabs.addTab(scrollarea, "Settings")

        # Master volume knob
        from collections import namedtuple
        LineInfo = namedtuple('LineInfo', ['widget','Interface'])

        # Volume Unactivating
        self.LineUnActivates = []
        p = LineInfo(widget.line1line2, BooleanControl(self.hw, self.hw.basepath+"/EAP/Settings/VolumeKnob/Line1Line2"))
        self.LineUnActivates.append(p)
        p = LineInfo(widget.line3line4, BooleanControl(self.hw, self.hw.basepath+"/EAP/Settings/VolumeKnob/Line3Line4"))
        self.LineUnActivates.append(p)
        p = LineInfo(widget.line5line6, BooleanControl(self.hw, self.hw.basepath+"/EAP/Settings/VolumeKnob/Line5Line6"))
        self.LineUnActivates.append(p)
        p = LineInfo(widget.line7line8, BooleanControl(self.hw, self.hw.basepath+"/EAP/Settings/VolumeKnob/Line7Line8"))
        self.LineUnActivates.append(p)

        for l in self.LineUnActivates:
            l.widget.setChecked(l.Interface.selected())
            l.widget.toggled.connect(l.Interface.select)

    def getDisplayTitle(self):
        return "M-Audio Profire 2626 and 610 Mixer"

#
# vim: et ts=4 sw=4
